#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author:skyoceanchen
# project_name:chengduairport
# py_name :processing_txt
# software: PyCharm
# datetime:2021/3/3 13:45
"""文件类工具"""
import datetime as dt
import imghdr
import os
import uuid

import filetype
from PIL import Image
from basic_type_operations.str_operation import StringOperation
from django.conf import settings
from django.http import StreamingHttpResponse
from django.utils.encoding import escape_uri_path  # 用于解决中文命名文件乱码问题

PassExPrentFile = ["png", "jpg", "doc", "pdf", "xlsx", 'docx', 'xls', 'zip', 'mp4']
passFile = ['.doc', '.docx']
picEx = ["png", "jpg"]


class FileOperation(object):
    @staticmethod
    def save_destination(dest, reqfile):
        if not os.path.exists(dest):
            if not isinstance(reqfile, bytes):
                with open(dest, "wb") as destination:
                    for chunk in reqfile.chunks():
                        destination.write(chunk)
            else:
                with open(dest, "wb") as destination:
                    destination.write(reqfile)

    @staticmethod
    def upload(FILES, file_type: str, **kwargs):
        """
            上传文件
        :return:
        """
        reqfile = FILES
        if kwargs:
            filename = kwargs['file_name']
            filename_old = kwargs['file_name']
        else:
            filename = reqfile.name
            filename_old = reqfile.name
        if str(filename).find('.') == -1:
            filename = filename + '.jpg'
            suffix = '.jpg'
        else:
            suffix = filename[filename.rfind('.'):]
        filename_old = filename
        if StringOperation.check_contain_zh_cn(filename) and imghdr.what(reqfile):
            filename_old = filename
            filename = str(uuid.uuid1()) + suffix

        current_time = dt.datetime.now().strftime("%Y%m%d%H%M%S")
        current_time_str = dt.datetime.now().strftime("%Y%m%d %H:%M:%S")

        pathdir = os.path.join(settings.MEDIA_ROOT, file_type, current_time).replace('\\', '/')
        pathRela = os.path.join(file_type, current_time, filename).replace('\\', '/')
        dest = os.path.join(settings.MEDIA_ROOT, file_type, current_time, filename).replace('\\', '/')

        if not os.path.exists(pathdir):
            os.makedirs(pathdir)
        thumb_flag = False
        if not isinstance(reqfile, bytes):
            if imghdr.what(reqfile):  # 是图片类型
                thumb_flag = True
                img_ = Image.open(reqfile).convert('RGB')
                width, height = img_.size
                size = (480, 480)
                img_.thumbnail(size, Image.ANTIALIAS)
                thumb_path = os.path.join(settings.MEDIA_ROOT, 'thumb', file_type, current_time).replace('\\', '/')
                if not os.path.exists(thumb_path):
                    os.makedirs(thumb_path)
                # print(thumb_path)
                # 保存
                img_.save(thumb_path + '/' + filename, quality=70)
        FileOperation.save_destination(dest, reqfile)
        res = {
            'original': settings.HTTP_HEAD + '/media/' + pathRela,
            'short_url': pathRela
        }
        if thumb_flag:
            res['thumb'] = settings.HTTP_HEAD + '/media/thumb/' + pathRela
        return res

    @staticmethod
    def file_upload(FILES, file_type: str, **kwargs):
        """
            上传文件
        :return:
        """
        try:
            reqfile = FILES
            filename = reqfile.name
            suffix = filename[filename.rfind('.'):]
            kind = filetype.guess(reqfile)
            if kind is None and suffix not in passFile:
                raise Exception('未知混乱或无效的文件')
            if kind and kind.extension not in PassExPrentFile:
                raise Exception('不允许的文件类型')
            current_time = dt.datetime.now().strftime("%Y%m%d%H%M%S")
            if StringOperation.check_contain_zh_cn(filename):
                filename = str(uuid.uuid1()) + suffix

            pathdir = os.path.join(settings.MEDIA_ROOT, file_type, current_time).replace('\\', '/')
            pathRela = os.path.join(file_type, current_time, filename).replace('\\', '/')
            dest = os.path.join(settings.MEDIA_ROOT, file_type, current_time, filename).replace('\\', '/')
            if not os.path.exists(pathdir):
                os.makedirs(pathdir)
            FileOperation.save_destination(dest, reqfile)
            res = {
                'original': settings.HTTP_HEAD + '/media/' + pathRela,
                'short_url': pathRela
            }
            return res
        except Exception as e:
            raise Exception('文件传输过程中发生异常，或者您上传了不允许的文件')

    @staticmethod
    def file_iterator(file_name):
        with open(file_name, 'rb') as f:
            while True:
                c = f.read()
                if c:
                    yield c
                else:
                    break

    @staticmethod
    def download(request, file_name, file_path):
        response = StreamingHttpResponse(FileOperation.file_iterator(file_path))
        file_end = file_path.split('.')[-1]
        agent = request.META.get('HTTP_USER_AGENT')
        if agent.upper().find("MSIE") != -1:
            response['Content-Disposition'] = "attachment; filename={0}".format(file_name + f'.{file_end}').encode(
                'gbk').decode('latin-1')
        elif agent.upper().find("EDGE") != -1:
            response['Content-Disposition'] = "attachment; filename={0}".format(file_name + f'.{file_end}').encode(
                'gb2312')
        elif agent.upper().find("TRIDENT") != -1:
            response['Content-Disposition'] = "attachment; filename={0}".format(file_name + f'.{file_end}').encode(
                'gb2312')
        else:
            response['Content-Disposition'] = 'attachment; filename={}'.format(
                escape_uri_path(file_name + f'.{file_end}'))
        response["Access-Control-Expose-Headers"] = "Content-Disposition"  # 为了使前端获取到Content-Disposition属性
        if file_end == "pdf":
            response["Content-type"] = "application/pdf"
        elif file_end == "zip":
            response["Content-type"] = "application/zip"
        elif file_end == "doc":
            response["Content-type"] = "application/msword"
        elif file_end == "xls":
            response["Content-type"] = "application/vnd.ms-excel"
        elif file_end == "xlsx":
            response["Content-type"] = "application/vnd.ms-excel"
        elif file_end == "docx":
            response["Content-type"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        elif file_end == "doc":
            response["Content-type"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        elif file_end == "ppt":
            response["Content-type"] = "application/vnd.ms-powerpoint"
        elif file_end == "pptx":
            response["Content-type"] = "application/vnd.openxmlformats-officedocument.presentationml.presentation"
        else:
            response['Content-Type'] = 'application/octet-stream'
        return response



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author:skyoceanchen
# project_name:automaticoffice
# py_name :异步管理文件上下文
# software: PyCharm
# datetime:2021/2/1 19:59

import asyncio

#
# class AContext:
#     def __init__(self):
#         print("in init")
#
#     async def __aenter__(self):
#         print("in aenter")
#
#     async def __aexit__(self, exc_type, exc_val, exc_tb):
#         print("in aexit", exc_type, exc_val, exc_tb)
#
#
# async def main():
#     async with AContext() as ac:
#         print("in with", ac)

'''
上面的代码通过使用asyncio中run_in_executor运行一个线程，来使得阻塞操作变为非阻塞操作，达到异步非阻塞的目的。
AsyncFile类提供了一些方法，这些方法将用于将write、read和readlines的调用添加到pending列表中。这些任务通过finally块中的事件循环在ThreadPoolExecutor进行调度。
yield 前半段用来表示__aenter__()
yield 后半段用来表示__aexit__()
使用finally以后可以保证链接资源等使用完之后能够关闭。
'''
# 3.7
from contextlib import asynccontextmanager
# 3.6
from async_generator import asynccontextmanager
from concurrent.futures.thread import ThreadPoolExecutor


# <editor-fold desc="写入">
class AsyncFile(object):
    def __init__(self, file, loop=None, executor=None):
        if not loop:
            loop = asyncio.get_running_loop()  # 获取当前运行事件循环
        if not executor:
            executor = ThreadPoolExecutor(10)  # 线程池数量10
        self.file = file
        self.loop = loop
        self.executor = executor
        self.pending = []
        self.result = []

    def write(self, string):
        """
        实现异步写操作
        :param string: 要写的内容
        :return:
        """
        self.pending.append(
            self.loop.run_in_executor(
                self.executor, self.file.write, string,
            )
        )

    def read(self, i):
        """
        实现异步读操作
        :param i:
        :return:
        """
        self.pending.append(
            self.loop.run_in_executor(self.executor, self.file.read, i, )
        )

    def readlines(self):
        self.pending.append(
            self.loop.run_in_executor(self.executor, self.file.readlines, )
        )


@asynccontextmanager
async def async_open(path, mode="w"):
    with open(path, mode=mode, encoding='utf8') as f:
        loop = asyncio._get_running_loop()
        file = AsyncFile(f, loop=loop)
        try:
            yield file
        finally:
            file.result = await asyncio.gather(*file.pending, loop=loop)


import tempfile
import os


async def main():
    # tempdir = tempfile.gettempdir()
    tempdir = os.getcwd()
    path = os.path.join(tempdir, "run.txt")
    print(f"临时文件目录:{path}")

    async with async_open(path, mode='w') as f:
        f.write("公众号: ")
        f.write("Python")
        f.write("学习")
        f.write("开发")


# </editor-fold>


# 同步挂起协程

# <editor-fold desc="读出">
class Sync():
    def __init__(self):
        self.pending = []
        self.finished = None

    def schedule_coro(self, coro, shield=True):
        # 如果去掉asyncio.shield，在取消fut函数的时候，就会导致coro协程也出错。
        fut = asyncio.shield(coro) if shield else asyncio.ensure_future(coro)
        self.pending.append(fut)
        return fut

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        # 退出async with的时候，任务列表中的任务进行并发操作。
        self.finished = await asyncio.gather(*self.pending, return_exceptions=True)


async def workload1():
    await asyncio.sleep(2)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 41")
    return 41


async def workload2():
    await asyncio.sleep(1)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 42")
    return 42


async def workload3():
    await asyncio.sleep(2)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 43")
    return 43


async def workload4():
    await asyncio.sleep(2)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 43")
    return 43


async def workload5():
    await asyncio.sleep(2)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 43")
    return 43


async def workload6():
    await asyncio.sleep(2)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 43")
    return 43


async def workload7():
    await asyncio.sleep(2)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 43")
    return 43


async def workload8():
    await asyncio.sleep(2)
    f = open('run.txt', 'r', encoding='utf8')
    data = f.read()
    print(data)
    print("These coroutines will be executed return 43")
    return 43


async def main2():
    async with Sync() as sync:
        # 使用异步上下文可以创建同步协程程序
        sync.schedule_coro(workload1())
        sync.schedule_coro(workload2())
        sync.schedule_coro(workload3())
        sync.schedule_coro(workload4())
        sync.schedule_coro(workload5())
        sync.schedule_coro(workload6())
        sync.schedule_coro(workload7())
        sync.schedule_coro(workload8())
    print("All scheduled corotines have retuned or throw:", sync.finished)


# </editor-fold>


if __name__ == '__main__':
    data = """
    E:\virtualenvs\automaticoffice\Scripts\python.exe F:/jiyiproj/automaticoffice/002common/文件以及文件夹/异步管理文件上下文.py
start
公众号: Python学习开发
These coroutines will be executed return 42
公众号: Python学习开发
These coroutines will be executed return 43
公众号: Python学习开发
These coroutines will be executed return 43
公众号: Python学习开发
These coroutines will be executed return 43
公众号: Python学习开发
These coroutines will be executed return 43
公众号: Python学习开发
These coroutines will be executed return 43
公众号: Python学习开发
These coroutines will be executed return 41
公众号: Python学习开发
These coroutines will be executed return 43
All scheduled corotines have retuned or throw: [41, 42, 43, 43, 43, 43, 43, 43]

Process finished with exit code 0
    """
    for i in range(1000):
        print(1)
        f = open('run.txt', 'a', encoding='utf8')
        f.write(data + '\n')
    print("start")
    # Python 3.6
    # asyncio.get_event_loop().run_until_complete(main())
    asyncio.get_event_loop().run_until_complete(main2())
    # Python 3.7
    # asyncio.run(main())



# -*- coding: utf-8 -*-
"""
@Time : 2023/8/16 21:11 
@Author : skyoceanchen
@TEL: 18916403796
@项目：文件使用
@File : doc_operation.by
@PRODUCT_NAME :PyCharm
"""

# from win32com import client as wc  # pip install pywin32
from win32com.client import Dispatch


class DocOperation(object):
    def __init__(self, path):
        self.path = path

    def read_doc(self):
        word = Dispatch('Word.Application')  # 打开word应用程序
        word.Visible = 0  # 后台运行,不显示
        word.DisplayAlerts = 0  # 不警告
        doc = word.Documents.Open(FileName=self.path, Encoding='gbk')
        text_list = []
        for para in doc.paragraphs:
            text = para.Range.Text.replace("\x15", "").replace("\x01", "").replace("\x0c", "").replace("\x07",
                                                                                                       "").replace(
                "\r", "").replace("\x0e", "")
            # text = ''.join(para.Range.Text.strip(" ").split('\r'))
            if text:
                text_list.append(text)
        doc.Close()
        word.Quit()
        text = "".join(text_list)
        return text



# -*- coding:utf-8 -*-
"""
@Time : 2023/3/22
@Author : skyoceanchen
@TEL: 18916403796
@File : excel_operation.py
@PRODUCT_NAME : PyCharm
"""
import pandas as pd
from basic_type_operations.list_operation import ListOperation
from openpyxl import Workbook, load_workbook
from openpyxl.cell import MergedCell, ReadOnlyCell
from openpyxl.drawing.image import Image
from openpyxl.styles import Alignment  # 对齐方式
from openpyxl.utils import FORMULAE  # 公式，但是可以检查公式的名称
import openpyxl
import datetime as dt
import os
from django.conf import settings
from openpyxl.styles import Border  # 边框
from openpyxl.styles import Side
from openpyxl.styles import Font  # 字体
from openpyxl.utils import get_column_letter, column_index_from_string  # 获取列的头 get_column_letter(a)=A

"""
=HYPERLINK("https://www.baidu.com","百度收缩")# 插入超链接
"""


class ExportXlsx(object):  # 导出excel 居中并自适应列宽
    def create_data_img_xlsx(self, data_list, file_type=None, ):
        wb = Workbook()
        sheet = wb.active
        row = len(data_list)
        col = len(data_list[0])
        for j in range(1, col + 1):
            self.cell_alignment(sheet.cell(1, j, value=data_list[0][j - 1]))
        column_letter_width = 25
        for i in range(2, row + 1):
            for j in range(1, len(data_list[i - 1]) + 1):
                value = data_list[i - 1][j - 1]
                column_letter = openpyxl.utils.cell.get_column_letter(j)
                if isinstance(value, str):
                    if value.endswith('png') or value.endswith('jpg') or value.endswith("PNG") or value.endswith("JPG"):
                        if os.path.exists(value):
                            location = f"{column_letter}{i}"
                            img = Image(value)
                            img_times = int(img.height / 90) + 1
                            height = int(img.height / img_times)
                            width = int(img.width / img_times)
                            image_size = (width, height,)
                            img.width, img.height = (width, height,)  # 这两个属性分别是对应添加图片的宽高
                            sheet.row_dimensions[i].height = height
                            sheet.column_dimensions[column_letter].width = column_letter_width  # 修改列的列宽
                            sheet.add_image(img, f'{location}', )  # 向单元格内指定添加图片
                    else:
                        self.cell_alignment(sheet.cell(i, j, value=value))
                        sheet.column_dimensions[column_letter].width = len(value.encode('utf-8'))
                else:
                    self.cell_alignment(sheet.cell(i, j, value=value))
                    # sheet.column_dimensions[column_letter].width = len(str(value).encode('utf-8'))

        current_time = dt.datetime.now().strftime("%Y%m%d%H%M%S")
        filename = f"{current_time}.xlsx"
        thumb_path = os.path.join(settings.MEDIA_ROOT, file_type).replace('\\', '/')
        dest = os.path.join(thumb_path, filename).replace('\\', '/')
        pathRela = os.path.join(file_type, filename).replace('\\', '/')
        if not os.path.exists(thumb_path):
            os.makedirs(thumb_path)
        wb.save(f'{dest}')
        return filename, dest, pathRela

    def create_data_xlsx(self, data_list, file_type=None, ):
        wb = Workbook()
        sheet = wb.active
        row = len(data_list)
        col = len(data_list[0])
        for i in range(1, row + 1):
            for j in range(1, col + 1):
                self.cell_alignment(sheet.cell(i, j, value=data_list[i - 1][j - 1]))
        self.auto_width(sheet)
        current_time = dt.datetime.now().strftime("%Y%m%d%H%M%S")
        filename = f"{current_time}.xlsx"
        thumb_path = os.path.join(settings.MEDIA_ROOT, file_type).replace('\\', '/')
        dest = os.path.join(thumb_path, filename).replace('\\', '/')
        pathRela = os.path.join(file_type, filename).replace('\\', '/')
        if not os.path.exists(thumb_path):
            os.makedirs(thumb_path)
        wb.save(f'{dest}')
        return filename, dest, pathRela

    # <editor-fold desc="自适应列宽">
    def auto_width(self, ws):
        lks = []  # 英文变量太费劲，用汉语首字拼音代替
        for i in range(1, ws.max_column + 1):  # 每列循环
            lk = 1  # 定义初始列宽，并在每个行循环完成后重置
            for j in range(1, ws.max_row + 1):  # 每行循环
                sz = ws.cell(row=j, column=i).value  # 每个单元格内容
                if isinstance(sz, str):  # 中文占用多个字节，需要分开处理
                    lk1 = len(sz.encode('utf-8'))  # gbk解码一个中文两字节，utf-8一个中文三字节，gbk合适
                else:
                    lk1 = len(str(sz))
                if lk < lk1:
                    lk = lk1  # 借助每行循环将最大值存入lk中
            lks.append(lk)  # 将每列最大宽度加入列表。（犯了一个错，用lks = lks.append(lk)报错，append会修改列表变量，返回值none，而none不能继续用append方法）
        # 第二步：设置列宽
        for i in range(1, ws.max_column + 1):
            k = get_column_letter(i)  # 将数字转化为列名,26个字母以内也可以用[chr(i).upper() for i in range(97, 123)]，不用导入模块
            ws.column_dimensions[k].width = lks[i - 1] + 2  # 设置列宽，一般加两个字节宽度，可以根据实际情况灵活调整

    # 有汉字的自适应
    def column_dimensions_width(self, sheet, data):
        """
        data 表格所有数据/部分数据/二维
        :param ws:
        :param data:
        :return:
        """
        lis = ListOperation.dimensional_list_two(data)
        for index, li in enumerate(lis):
            # len_data = []
            # for m in li:
            #     if isinstance(m, float):
            #         m = round(m, 3)
            #     len_data.append(len(str(m)))
            # max_len = max(len_data)
            # ws.column_dimensions[get_column_letter(index + 1)].width = max_len * 2
            df = pd.DataFrame(li)
            # # ws自动设置列宽
            df_len = df.apply(lambda x: [(len(str(i).encode('utf-8')) - len(str(i))) / 2 + len(str(i)) for i in li],
                              axis=0)
            df_len_max = df_len.apply(lambda x: max(x), axis=0)
            column_letter = get_column_letter(index + 1)
            # 列的宽度
            columns_length = (len(str(index).encode('utf-8')) - len(str(index))) / 2 + len(str(index))
            data_max_length = df_len_max[0]
            column_width = [data_max_length if columns_length < data_max_length else columns_length][0]
            column_width = [column_width if column_width <= 50 else 50][0] + 3  # 列宽不能超过50
            # 更改列的宽度
            sheet.column_dimensions['{}'.format(column_letter)].width = column_width
        return sheet

    # </editor-fold>
    # <editor-fold desc="对齐方式-水平居中-垂直居中">
    def cell_alignment(self, cell):
        """
        horizontal_alignments = (
            "general:上", "left", "center", "right", "fill：满", "justify：两端对齐", "centerContinuous：中心连续",
            "distributed：分散",)
        vertical_aligments = ("top", "center", "bottom", "justify", "distributed",)
        """
        alignment = Alignment(
            # ("general", "left", "center", "right", "fill", "justify", "centerContinuous", "distributed",)
            horizontal='center',  # 水平居中
            # ( "top", "center", "bottom", "justify", "distributed",)
            vertical='center',  # 垂直居中
            textRotation=0,  # 文字旋转range(181) 0-180°
            wrapText=True,  # 自动换行 True/Flase
            shrinkToFit=True,  # 是否适应列宽 True/Flase
            indent=0,  # 缩进
            relativeIndent=0,  # 相对缩进
            justifyLastLine=None,  # 证明最后一行 True/Flase
            readingOrder=0,  # 阅读顺序
        )
        cell.alignment = alignment
        return cell

    # </editor-fold>
    def read_xlsx(self, path):
        wb = load_workbook(path, read_only=True, data_only=True)
        ws = wb.active
        max_column = ws.max_column
        max_row = ws.max_row
        data_lis = []
        for index, i in enumerate(ws):
            # if index > 0:
            lis = [j.value for j in i]
            # lis[1] = dt.datetime.strptime(lis[1], "%Y-%m-%d %H:%M:%S")
            data_lis.append(lis)
        return data_lis


# openpyxl操作
class OpenpyxlCell(object):
    # <editor-fold desc="边框设置">
    def cell_border(self, cell):
        """
            style = NoneSet(values=('dashDot','dashDotDot', 'dashed','dotted',
                                'double','hair', 'medium', 'mediumDashDot', 'mediumDashDotDot',
                                'mediumDashed', 'slantDashDot', 'thick', 'thin')
                        )
        :param cell:
        :return:
        """
        border = Border(
            left=Side(style='thin', color='000000'),
            right=Side(style='thin', color='000000'),
            top=Side(style='thin', color='000000'),
            bottom=Side(style='thin', color='000000'),
            diagonal=Side(style='thin', color='000000'),
            diagonal_direction=0,  # 对角线方向
            vertical=Side(style='thin', color='000000'),
            horizontal=Side(style='thin', color='000000'),
            diagonalUp=False,  # True/False
            diagonalDown=False,  # True/False
            # outline=Side(style='medium', color='000000'),
            outline=True,  # True/False
            start=None,  # Side(style='thin', color='000000')
            end=None)  # Side(style='thin', color='000000')
        cell.border = border
        return cell

    # </editor-fold>
    # <editor-fold desc="文字设置">
    def cell_font(self, cell):
        """
        # 默认字体
        DEFAULT_FONT = Font(name="Calibri", sz=11, family=2, b=False, i=False,
                        color=Color(theme=1), scheme="minor")
        :param cell:
        :return:
        """
        font = Font(
            name='宋体',  # 字体名字/Calibri/宋体
            family=None,  # min=0, max=14
            size=30,  # 字体大小
            bold=True,  # 是否加粗
            italic=True,  # 斜体
            underline='none',  # 下划线 ('single：单线', 'double：双线', 'singleAccounting','doubleAccounting')
            color='000000',  # 颜色,可以用colors中的颜色
            charset=None,  # 字符集；字元集
            scheme=None,  # 计划 概形 方案  ("major":主要的, "minor":次要)
            strikethrough=None,  # 删除线，加删除线
            vertAlign=None,  # 纵向对齐方式('superscript:上标 标在上方的附加字', 'subscript:脚注', 'baseline:底线')
            outline=None,  # 轮廓 轮廓线 大纲
            shadow=None,  # 阴影 影子 影
            condense=None,  # 压缩 浓缩 紧缩
            extend=None  # 延伸 延长 扩展
        )
        cell.font = font
        return cell

    # </editor-fold>
    # <editor-fold desc="对齐方式-水平居中-垂直居中">
    def cell_alignment(self, cell):
        horizontal_alignments = (
            "general:上", "left", "center", "right", "fill：满", "justify：两端对齐", "centerContinuous：中心连续",
            "distributed：分散",)
        vertical_aligments = ("top", "center", "bottom", "justify", "distributed",)
        alignment = Alignment(
            # ("general", "left", "center", "right", "fill", "justify", "centerContinuous", "distributed",)
            horizontal='center',  # 水平居中
            # ( "top", "center", "bottom", "justify", "distributed",)
            vertical='center',  # 垂直居中
            textRotation=0,  # 文字旋转range(181) 0-180°
            wrapText=True,  # 自动换行 True/Flase
            shrinkToFit=True,  # 是否适应列宽 True/Flase
            indent=0,  # 缩进
            relativeIndent=0,  # 相对缩进
            justifyLastLine=None,  # 证明最后一行 True/Flase
            readingOrder=0,  # 阅读顺序
        )
        cell.alignment = alignment
        return cell

    # </editor-fold>


class openpyxlSheet(object):
    # <editor-fold desc="工作表保护 不可更改被保护的工作表">
    def password_sheet(self, sheet, enable=True, password=None):
        """

        :param ws:
        :param enable: True保护/Flase 不保护
        :param password:str 密码
        :return:
        """
        # 工作表保护 不可更改被保护的工作表
        # 也可以通过在openpyxl.worksheet.protection.SheetProtection对象上设置属性来锁定工作表的各个方面 。
        # 与工作簿保护不同，可以使用或不使用密码来启用工作表保护。
        # 使用openpxyl.worksheet.protection.SheetProtection.sheet属性或调用enable（）或disable（）可启用工作表保护 ：
        # 如果未指定密码，则用户无需指定密码即可禁用配置的工作表保护。
        # 否则，他们必须提供密码才能更改已配置的保护。
        # 使用openpxyl.worksheet.protection.SheetProtection.password()属性设置密码
        if enable:
            sheet.protection.password = password
            sheet.protection.sheet = True
            sheet.protection.enable()
        else:
            sheet.protection.password = password
            sheet.protection.sheet = True
            sheet.protection.disable()

    # </editor-fold>
    # <editor-fold desc="自适应列宽">
    def auto_width(self, sheet):
        lks = []  # 英文变量太费劲，用汉语首字拼音代替
        for i in range(1, sheet.max_column + 1):  # 每列循环
            lk = 1  # 定义初始列宽，并在每个行循环完成后重置
            for j in range(1, sheet.max_row + 1):  # 每行循环
                sz = sheet.cell(row=j, column=i).value  # 每个单元格内容
                if isinstance(sz, str):  # 中文占用多个字节，需要分开处理
                    lk1 = len(sz.encode('utf-8'))  # gbk解码一个中文两字节，utf-8一个中文三字节，gbk合适
                else:
                    lk1 = len(str(sz))
                if lk < lk1:
                    lk = lk1  # 借助每行循环将最大值存入lk中
            lks.append(lk)  # 将每列最大宽度加入列表。（犯了一个错，用lks = lks.append(lk)报错，append会修改列表变量，返回值none，而none不能继续用append方法）
        # 第二步：设置列宽
        for i in range(1, sheet.max_column + 1):
            k = get_column_letter(i)  # 将数字转化为列名,26个字母以内也可以用[chr(i).upper() for i in range(97, 123)]，不用导入模块
            sheet.column_dimensions[k].width = lks[i - 1] + 2  # 设置列宽，一般加两个字节宽度，可以根据实际情况灵活调整

    # </editor-fold>


class OpenpyxlOperation(openpyxlSheet, OpenpyxlCell):
    def __init__(self, path, read_only=False,
                 data_only=False, keep_links=True, rich_text=False, read=True, write=False, read_Write=None,
                 keep_vba=False,
                 sheet_title=None, tabColor=None):
        """
        :param filename:
        :param read_only:bool 优化阅读，内容不能编辑
        :param keep_vba:bool 控制是否保留任何Visual Basic元素（默认）。如果保留它们，则它们仍不可编辑
        :param data_only:bool 控制具有公式的单元格是否具有公式（默认值）或Excel上次读取工作表时存储的值。
        :param keep_links:bool 是否保留到外部工作簿的链接。默认值为True
        :param sheet_title:
        :param tabColor:
         """
        self.path = path
        self.read = read
        self.write = write
        self.read_only = read_only
        self.data_only = data_only
        self.keep_vba = keep_vba
        self.keep_links = keep_links
        self.rich_text = rich_text
        self.active()

    def get_sheet_title(self):
        return self.wb.sheetnames

    def active(self):
        # 阅读现有的工作簿
        if self.read:
            self.wb = load_workbook(self.path, read_only=False,
                                    data_only=False, keep_links=True, rich_text=False)

        elif self.write:
            self.wb = Workbook()
            # if sheet_title:
            #     ws.title = sheet_title
            # if tabColor:
            #     ws.sheet_properties.tabColor = tabColor
        sheet = self.wb.active
        return sheet

    # <editor-fold desc="选择一个工作表">
    def select_sheet(self, sheet_title):
        sheet = self.wb[sheet_title]
        return sheet

    # </editor-fold>
    # <editor-fold desc="创建新的工作表">
    def create_sheet(self, sheet_name, index=None):
        """
        :param sheet_name:
        :param index: None # 在末尾插入(默认) 0  # 第一个位置插入 -1 # 插入到倒数第二位置
        :return:
        """
        sheet = self.wb.create_sheet(title=sheet_name, index=index)
        return sheet

    # </editor-fold>
    # <editor-fold desc="单个工作簿中创建工作表的副本:复制工作表">
    def copy_worksheet(self, sheet):
        """
        仅复制单元格（包括值，样式，超链接和注释）和某些工作表属性（包括尺寸，格式和属性）。不复制所有其他工作簿/工作表属性-例如图像，图表。

您也不能在工作簿之间复制工作表。如果工作簿以只读或仅写 模式打开，则不能复制工作表。
        :param source:
        :return:
        """
        self.wb.copy_worksheet(sheet)

    # </editor-fold>
    # <editor-fold desc="获取所有的工作表名">
    def get_all_sheet(self, ):
        return self.wb.sheetnames

    # </editor-fold>
    # <editor-fold desc="取值:根据位置取值">
    def get_sheet_value(self, sheet, location, get_value=True):
        if get_value:
            if isinstance(location, int):
                # 返回第一行,对象
                return [i.value for i in sheet[location]]
            elif location[-1].isdigit():
                return sheet[location].value
            else:
                # 返回第一列 数据
                return [i.value for i in sheet[location]]
        else:
            if isinstance(location, int):
                # 返回第一行,对象
                return list(sheet[location])
            elif location[-1].isdigit():
                return sheet[location]
            else:
                # 返回第一列 数据
                return list(sheet[location])

    # </editor-fold>
    # <editor-fold desc="赋予值/修改值">
    def post_sheet_value(self, sheet, location, value):
        sheet[location] = value

    # </editor-fold>
    # <editor-fold desc="cell赋予值">
    def cell(self, sheet, value=None, top_distance=0, left_distance=0):
        for row in range(len(value)):
            for column in range(len(value[row])):
                sheet.cell(row=row + 1 + top_distance, column=column + 1 + left_distance, value=value[row][column])

    # </editor-fold>
    # <editor-fold desc=" col的表格列名：AA列 A列">
    def get_column_letter(self, col):
        return get_column_letter(col)

    # </editor-fold>
    # <editor-fold desc="合并/取消合并单元格">
    # 合并单元格
    def merge_cells(self, sheet, left_col_index=None, right_col_index=None, start_row=None,
                    start_column=None, end_row=None, end_column=None, range_string=None):
        """
        sheet.merge_cells('A2:D2')
        # or equivalently
        sheet.merge_cells(start_row=2, start_column=1, end_row=4, end_column=4)
        :return:
        """
        if left_col_index and left_col_index:
            sheet.merge_cells(range_string='%s:%s' % (left_col_index, right_col_index))
        elif start_row and start_column and end_row and end_column:
            sheet.merge_cells(start_row=start_row, start_column=start_column, end_row=end_row,
                              end_column=end_column)
        return sheet

    # 取消合并单元格
    def unmerge_cells(self, sheet, left_col_index=None, right_col_index=None, start_row=None,
                      start_column=None, end_row=None, end_column=None, range_string=None, ):
        """
        sheet.unmerge_cells('A2:D2')
        sheet.unmerge_cells(start_row=2, start_column=1, end_row=4, end_column=4)
        :return:
        """
        if left_col_index and left_col_index:
            sheet.unmerge_cells(range_string='%s:%s' % (left_col_index, right_col_index))
        elif start_row and start_column and end_row and end_column:
            sheet.unmerge_cells(start_row=start_row, start_column=start_column, end_row=end_row,
                                end_column=end_column)
        return sheet

    # </editor-fold>
    # <editor-fold desc="合并单元格并居中">
    def merge_cells_alignment(self, sheet, left_col_index=None, right_col_index=None, start_row=None,
                              start_column=None,
                              end_row=None, end_column=None):
        """
        range_string:'A2:D2'
        ws.merge_cells('A2:D2')
        # or equivalently
        ws.merge_cells(start_row=2, start_column=1, end_row=4, end_column=4)
        :return:
        """
        if left_col_index and left_col_index:
            sheet.merge_cells(range_string='%s:%s' % (left_col_index, right_col_index))
            cell = sheet[left_col_index]
        elif start_row and start_column and end_row and end_column:
            sheet.merge_cells(start_row=start_row, start_column=start_column, end_row=end_row,
                              end_column=end_column)
            cell = sheet.cell(start_row, start_column)
        else:
            cell = False
        self.cell_alignment(cell)
        return sheet

    # </editor-fold>
    # <editor-fold desc="添加图片">
    def add_image(self, sheet, img_path, location):
        """

        :param sheet:
        :param img_path:
        :param location:可选地为左上角的锚提供一个单元格
        :return:
        """
        img = Image(img_path)
        sheet.add_image(img=img, anchor=location)

    # </editor-fold>
    # <editor-fold desc="折叠（轮廓）行 列">
    # 列折叠
    def column_dimensions(self, sheet, left_col_index=None, right_col_index=None, outline_level=1, hidden=False):
        sheet.column_dimensions.group(left_col_index, right_col_index, outline_level=outline_level, hidden=hidden)

    # 行折叠
    def row_dimensions(self, sheet, left_row_index, right_row_index, outline_level=1, hidden=False):
        sheet.row_dimensions.group(left_row_index, right_row_index, outline_level=outline_level, hidden=hidden)

    # </editor-fold>
    # <editor-fold desc="求和">
    def sum(self, sheet, location, left_index, right_index):
        sheet[location] = "=SUM(%s,%s)" % (left_index, right_index)
        return sheet

    # </editor-fold>
    # <editor-fold desc="工作簿保护">
    def sheet_name_password(self, workbookPassword=None, lockStructure=True):
        # 为防止其他用户查看隐藏的工作表，添加，移动，删除或隐藏工作表以及重命名工作表，
        # 可以使用密码保护工作簿的结构。可以使用openpyxl.workbook.protection.WorkbookProtection.workbookPassword()属性设置密码
        self.wb.security.workbookPassword = workbookPassword
        self.wb.security.lockStructure = lockStructure
        # 工作簿保护

        # # 同样，可以通过设置另一个密码来防止从共享工作簿中删除更改跟踪和更改历史记录。
        # # 可以使用openpyxl.workbook.protection.WorkbookProtection.revisionsPassword()属性设置此密码
        # wb.security.revisionsPassword = '123'
        # # openpyxl.workbook.protection.WorkbookProtection对象上的其他属性精确地控制了什么限制，
        # # 但是只有设置了适当的密码后，这些属性才会强制实施。
        # # 如果需要在不使用默认哈希算法的情况下设置原始密码值，则提供特定的设置器功能-例如
        # hashed_password = 123
        # wb.security.set_workbook_password(hashed_password, already_hashed=True)

    # </editor-fold>


class OpenpyxlReadOperation(OpenpyxlOperation):

    def read_xlsx(self, sheet):
        max_column = sheet.max_column
        max_row = sheet.max_row
        data_lis = []
        for index, i in enumerate(sheet):
            # if index > 0:
            lis = [j.value for j in i]
            # lis[1] = dt.datetime.strptime(lis[1], "%Y-%m-%d %H:%M:%S")
            data_lis.append(lis)
        return data_lis

    # 取消合并行列并每各分开值相同
    def get_excel(self, sheet_name, to_path):
        sheet = self.select_sheet(sheet_name)

        # 获取单元格的值，若为合并单元格，则返回合并区域左上角的值
        def get_cell_val(i, j):
            cell = sheet.cell(i, j)
            # 判断单元格类型是否为合并单元格
            if isinstance(cell, MergedCell):
                for merged_range in sheet.merged_cells.ranges:
                    if cell.coordinate in merged_range:
                        cell = sheet.cell(row=merged_range.min_row, column=merged_range.min_col)
            return cell.value

        rows = sheet.max_row
        cols = sheet.max_column
        pd_lis = []
        for row in range(1, rows + 1):
            lis = []
            for col in range(1, cols + 1):
                lis.append(get_cell_val(row, col))
            pd_lis.append(lis)
        pf = pd.DataFrame(pd_lis)
        pf.to_excel(to_path, index=False, header=False)


class OpenpyxlWriteOperation(OpenpyxlOperation):

    def write(self, sheet, lis):
        for row in lis:
            sheet.append(row)

    def save(self, path):
        self.wb.save(path)



# -*- coding: utf-8 -*-
"""
@Time : 2023/8/11 17:28
@Author : skyoceanchen
@TEL: 18916403796
@项目：file_operations
@File : pickle_operation.by
@PRODUCT_NAME :PyCharm
"""

import os
# ，还有跟python2它完全一样的叫做 cpickle，两者的区别就是后者更快。所以，下面操作中，不管是用import pickle ，还是用import cpickle as pickle ，在功能上都是一样的
import pickle
import shelve


class PickleOperation(object):
    def __init__(self, path):
        self.path = path

    def write(self, data):
        """
        :param data: { "a":[1, 2, 3, 4, 5]}
        :return:
        """
        f = open(self.path, "wb")
        # pickle.dump(data, f)  ##文件中以 ascii 格式保存数据
        # 二进制存储文件更小
        pickle.dump(data, f, True)  ##文件中以二进制格式保存数据
        f.close()

    def read(self):
        return pickle.load(open(self.path, "rb"))

    def st_size(self, path1, path2):
        s1 = os.stat(path1).st_size
        s2 = os.stat(path2).st_size
        return s1, s2


"""
利用shelve 模块，你可以将Python 程序中的变量保存到二进制的shelf 文件中。
这样，程序就可以从硬盘中恢复变量的数据。shelve 模块让你在程序中添加“保存”
和“打开”功能。例如，如果运行一个程序，并输入了一些配置设置，就可以将这
些设置保存到一个shelf 文件，然后让程序下一次运行时加载它们。
"""
"""
在Windows 上运行前面的代码，你会看到在当前工作目录下有3 个新文件：
mydata.bak、mydata.dat 和mydata.dir。在OS X 上，只会创建一个mydata.db 文件。
"""
"""
shelf 值不必用读模式或写模式打开，因为它们在打开后，既能读又能写
"""


class ShelveOperation(object):
    def __init__(self, path_name):
        self.path_name = path_name
        self.shelve_file = shelve.open(self.path_name)

    def write(self, data):
        self.shelve_file['key'] = data
        self.close()

    def read(self):
        data = self.shelve_file['key']
        self.close()
        return data

    def close(self):
        self.shelve_file.close()

    # 编码
    def keyencoding(self):
        return self.shelve_file.keyencoding



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author:skyoceanchen
# project_name:chengduairport
# py_name :processing_txt
# software: PyCharm
# datetime:2021/3/3 13:45
"""文件类工具"""
import datetime as dt
import glob
import linecache
import math
import os
import platform
import time
from pathlib import Path
from shutil import copy2, rmtree

import pandas as pd
from basic_type_operations.date_operation import DateOperation
from pywintypes import Time  # 可以忽视这个 Time 报错（运行程序还是没问题的）
from win32file import CreateFile, SetFileTime, CloseHandle
from win32file import GENERIC_READ, GENERIC_WRITE, OPEN_EXISTING


class FileBasicOperation(object):
    # <editor-fold desc="删除文件或者文件夹">
    def remove_path(self, path):
        """
        :param path: 文件或者文件夹路径,文件架空不空都强删
        :return:
         os.remove#删除文件
        os.rmdir#删除文件夹（只能删除空文件夹）
        os.removedirs#移除目录(必须是空目录)
        shutil.rmtree#删除文件夹
        """
        if os.path.isfile(path):
            os.remove(path)
        else:
            rmtree(path)

    def delete(self, path):
        FileBasicOperation().remove_path(path)

    # </editor-fold>
    # <editor-fold desc="创建文件夹">
    def makedirs(self, path):
        path_obj = Path(path)
        if not path_obj.suffix and not path_obj.exists():
            path_obj.absolute().mkdir(parents=True)
        elif path_obj.suffix and not path_obj.absolute().parent.exists():
            path_obj.absolute().parent.mkdir(parents=True)
        return path_obj.absolute()

    # </editor-fold>
    # <editor-fold desc="获取桌面路径">
    # 这样做的好处是可以把数据放在桌面上，在不同的电脑上都能调用代码对数据进行处理。
    # 如果是在一条电脑上把桌面路径固定在字符串中，则换一台电脑就必须修改桌面路径。
    def get_desktop_path(self, ):
        return os.path.join(os.path.expanduser("~"), 'Desktop')

    # </editor-fold>
    # <editor-fold desc="获取文件的大小,结果保留两位小数，单位为MB">
    @staticmethod
    def file_size(file):
        fsize = os.path.getsize(file)
        fsize = fsize / float(1024 * 1024)
        return round(fsize, 2)

    # </editor-fold>
    # <editor-fold desc="获取文件的访问时间">
    @staticmethod
    def file_access_time(file):
        t = os.path.getatime(file)
        return DateOperation.TimeStampToTime(t)

    # </editor-fold>
    # <editor-fold desc="获取文件的创建时间">
    @staticmethod
    def file_create_time(file):
        t = os.path.getctime(file)
        return DateOperation.TimeStampToTime(t)

    # </editor-fold>
    # <editor-fold desc="获取文件的修改时间">
    @staticmethod
    def file_modify_time(file):
        t = os.path.getmtime(file)
        return DateOperation.TimeStampToTime(t)

    # </editor-fold>
    # <editor-fold desc="获取文件时间">
    def get_file_time(self, file):
        from win32file import CreateFile, GetFileTime, CloseHandle
        from win32file import GENERIC_READ, GENERIC_WRITE, OPEN_EXISTING
        fh = CreateFile(file, GENERIC_READ | GENERIC_WRITE, 0, None, OPEN_EXISTING, 0, 0)
        createTimes, accessTimes, modifyTimes = GetFileTime(fh)
        CloseHandle(fh)
        return createTimes, accessTimes, modifyTimes

    # </editor-fold>
    # <editor-fold desc="修改文件时间">
    def modify_file_time(self, file, create_time=None, modify_time=None, access_time=None, offset=(0, 0, 0),
                         format="%Y-%m-%d %H:%M:%S"):

        if not create_time:
            create_time = FileBasicOperation.file_create_time(file)
        if not modify_time:
            modify_time = FileBasicOperation.file_modify_time(file)
        if not access_time:
            access_time = FileBasicOperation.file_access_time(file)
        fh = CreateFile(file, GENERIC_READ | GENERIC_WRITE, 0, None, OPEN_EXISTING, 0, 0)
        createTimes = Time(time.mktime(DateOperation().timeOffsetAndStruct(create_time, format, offset[0])))
        modifyTimes = Time(time.mktime(DateOperation().timeOffsetAndStruct(modify_time, format, offset[1])))
        accessTimes = Time(time.mktime(DateOperation().timeOffsetAndStruct(access_time, format, offset[2])))
        SetFileTime(fh, createTimes, accessTimes, modifyTimes)
        CloseHandle(fh)

    # </editor-fold>
    # <editor-fold desc="获取文件某一行/文件内容">
    @staticmethod
    def getlines(file, line=None):
        """
        :param file: 目标文件
        :param line: 行
        :return:
        """
        if not line:
            return linecache.getline(file, line)
        else:
            return linecache.getlines(file)

    # </editor-fold>
    # <editor-fold desc="读取文件最后n行">
    @staticmethod
    def get_last_lines(file, n):
        blk_size_max = 4096
        n_lines = []
        with open(file, 'rb') as fp:
            fp.seek(0, os.SEEK_END)
            cur_pos = fp.tell()
            while cur_pos > 0 and len(n_lines) < n:
                blk_size = min(blk_size_max, cur_pos)
                fp.seek(cur_pos - blk_size, os.SEEK_SET)
                blk_data = fp.read(blk_size)
                assert len(blk_data) == blk_size
                lines = blk_data.decode().split('\r\n')
                # adjust cur_pos
                if len(lines) > 1 and len(lines[0]) > 0:
                    n_lines[0:0] = lines[1:]
                    cur_pos -= (blk_size - len(lines[0]))
                else:
                    n_lines[0:0] = lines
                    cur_pos -= blk_size
                fp.seek(cur_pos, os.SEEK_SET)
        if len(n_lines) > 0 and len(n_lines[-1]) == 0:
            del n_lines[-1]
        return n_lines[-n:]

    # </editor-fold>
    # <editor-fold desc="扫描输出所有的子目录（子文件夹）">
    # 扫描输出所有的子目录（子文件夹）
    # 使用os.walk输出某个目录下的所有文件   er
    @staticmethod
    def dir_son_names(dir_path):
        """
        :param dir_path: 目标文件夹
        :return:
        """
        lis = list()
        for curDir, dirs, files in os.walk(dir_path):
            """
            print("现在的目录：", curDir)
            print("该目录下包含的子目录：", str(dirs))
            print("该目录下包含的文件：", str(files))
            """
            for _dir in dirs:
                lis.append(os.path.join(curDir, _dir))
        return lis

    # </editor-fold>
    # <editor-fold desc="输出所有文件">
    # 扫描输出所有文件的路径
    @staticmethod
    def file_paths(dir_path):
        """

        :param dir_path: 目标文件夹
        :return:
        """
        lis = list()
        for curDir, dirs, files in os.walk(dir_path):
            for file in files:
                lis.append(os.path.join(curDir, file))
        return lis

    # </editor-fold>
    # <editor-fold desc="输出指定类型文件">
    @staticmethod
    def file_endswith_path(dir_path, endswith):
        """

        :param dir_path:目录地址
        :param endswith: 文件结尾
        :return:
        """
        lis = list()
        for curDir, dirs, files in os.walk(dir_path):
            lis.append([os.path.join(curDir, file) for file in files if file.endswith(endswith)])
        res = (x for j in lis for x in j)
        return list(res)

    # </editor-fold>
    # <editor-fold desc="文件去重-相同日期文件">
    @staticmethod
    def file_qch_date(dir_path):
        wj_names = os.listdir(dir_path)
        wj_list = []
        num = 0
        for wj in wj_names:
            new_wj = wj[:-11]
            if new_wj not in wj_list:
                wj_list.append(new_wj)
            else:
                os.remove(dir_path + "\\" + wj)
                num += 1
        return num

    # </editor-fold>
    # <editor-fold desc="找指定时间段条件的文件">
    @staticmethod
    def file_move(dir_path, start_date, end_date):
        # 本文件移动至对应新建文件夹，非本月文件直接删除
        """
        :param file_dir_path: 目标文件夹
        :return:
        """
        # # 日期格式：xxxx-xx eg:2020-07-01
        # # 生成日期区间-字符串类型
        date_xl_str = [str(i)[:10] for i in pd.date_range(start_date, end_date, freq='D')]
        # # # 创建指定文件夹
        _new_dir_path = os.getcwd() + '\\' + start_date + "~" + end_date
        try:
            os.mkdir(_new_dir_path)
        except:
            pass
        # time_data = []
        for curDir, dirs, files in os.walk(dir_path):
            for file in files:
                old_dir_path = os.path.join(curDir, file)
                # time_data.append(str(time.strftime("%Y-%m-%d", time.localtime(os.path.getctime(old_dir_path)))))
                new_dir_path = os.path.join(_new_dir_path, file)
                # file_date = file.split("_")[-1][:10]
                # 文件创建时间
                file_date = str(time.strftime("%Y-%m-%d", time.localtime(os.path.getctime(old_dir_path))))
                # print(old_dir_path, '*******', new_dir_path, '*******', file_date)
                try:
                    # os.rename(old_dir_path, new_dir_path) if file_date in date_xl_str else os.remove(old_dir_path)
                    if file_date in date_xl_str:
                        # os.rename(old_dir_path, new_dir_path)  # 把文件移动到另外一个文件夹
                        copy2(old_dir_path, new_dir_path)
                except Exception as e:
                    # os.remove(old_dir_path)
                    pass

    # </editor-fold>
    # <editor-fold desc="文件或者文件夹-重新命名">
    @staticmethod
    def path_renames(old_path_name, new_path_name):
        # os.rename(old_file_path, new_file_path)# 只能对相应的文件进行重命名, 不能重命名文件的上级目录名.
        try:
            # 是os.rename的升级版, 既可以重命名文件, 也可以重命名文件的上级目录名
            os.renames(old_path_name, new_path_name)
            return True
        except Exception as e:
            return False

    # </editor-fold>
    # <editor-fold desc="创建具有顺序的文件夹或者文件">
    @staticmethod
    def file_number(cap_num, path_name_end=None, to_dir=None):
        """
        :param cap_num:创建个数
        :param path_name_end: 文件的后缀名 如txt, 不传创建文件夹
        :param to_dir: 存储路径
        :return:
        """
        try:
            cap_num_t = cap_num
            cap_count = 0
            while cap_num:
                cap_count = cap_count + 1
                cap_num = math.floor(cap_num / 10)
            fix = '%0' + str(cap_count) + 'd'  # 得到图片保存的前缀，比如001.png，0001.png
            cap_cnt = 1
            if not os.path.exists(to_dir):
                os.makedirs(to_dir)
            while cap_num_t:
                if path_name_end:
                    if to_dir:
                        path = os.path.join(to_dir, str(fix % cap_cnt) + '.' + path_name_end)
                    else:
                        path = str(fix % cap_cnt) + '.' + path_name_end
                    with open(path, mode="w", encoding="utf-8") as f:  # 写文件,当文件不存在时,就直接创建此文件
                        pass
                else:
                    if to_dir:
                        path = os.path.join(to_dir, str(fix % cap_cnt))
                    else:
                        path = str(fix % cap_cnt)
                    os.makedirs(path)
                cap_cnt = cap_cnt + 1
                cap_num_t -= 1
            return True
        except Exception as e:
            return False

    # </editor-fold>
    # <editor-fold desc="生成以时间戳和文件名称命名的文件或文件夹">
    @staticmethod
    def file_datetime(name_start=None, to_Dir=None):
        """
        :param name_start: 文件名或者文件名称 a.txt a
        :param rootDir: 存储路径
        :return:
        """
        name_end = None
        if '.' in name_start:
            name_end = name_start.split('.')[1]
            name_start = name_start.split('.')[0]
        date = dt.datetime.now().strftime('%Y%m%d%H%M%S%f')
        if name_end:
            if to_Dir:
                path = os.path.join(to_Dir, str(name_start) + str(date) + '.' + name_end)
            else:
                path = str(name_start) + str(date) + '.' + name_end
            f = open(path, 'w')
            f.close()
        else:
            if name_start:
                if to_Dir:
                    path = os.path.join(to_Dir, str(name_start) + str(date))
                else:
                    path = str(name_start) + str(date)
            else:
                if to_Dir:
                    path = os.path.join(to_Dir, str(date))
                else:
                    path = str(date)
            os.mkdir(path)
        return path

    # </editor-fold>
    # <editor-fold desc="过滤指定文件">
    @staticmethod
    def filter_files(file_list, endswith):
        def is_index(n):
            if n.endswith(endswith):
                return n

        file_list = list(filter(is_index, file_list))
        return file_list

    # </editor-fold>
    # <editor-fold desc="文件排序">
    @staticmethod
    def file_sort_absolute(dir_path, endswith):
        """
        :param path:文件位置
        :param endswith: 文件以什么结尾比如py，xls,word,txt等
        :param absolute: True是返回绝对路径，Flase是返回相对路径
        :return: 列表内都是文件的绝对路径
        """
        file_list = os.listdir(dir_path)
        length = len(endswith) + 1
        file_list = FileBasicOperation.filter_files(file_list, endswith)
        file_list.sort(key=lambda x: int(x[:-length]) if x.endswith(endswith) else False)
        file_list = [os.path.join(dir_path, i) for i in file_list]
        return file_list

    # </editor-fold>
    # <editor-fold desc="不全是num的文件数字排序">
    @staticmethod
    def file_other_sort(dir_path, endswith, split=None, location=None):
        """
        :param path:文件位置
        :param endswith: 文件以什么结尾比如py，xls,word,txt等
        :param split: split 右边是数字
        :param location:
        需要配合 split进行使用
        right 右边是数字 left 左边是数字
        1 拆分后索引1的位置是数字，2 拆分后索引2的位置是数字 .。。
        ‘create’ 按照文件创建时间进行排序
        ‘modify’ 按照文件修改时间进行排序
        ‘access’ 按照文件访问时间进行排序
        None 默认排序
        :param absolute: True是返回绝对路径，Flase是返回相对路径
        :return: 列表内都是文件的绝对路径
        """
        file_list = os.listdir(dir_path)
        file_list = FileBasicOperation.filter_files(file_list, endswith)
        length = len(endswith) + 1
        if split:
            if location not in ['right', "left", "create", "modify", "access", None] and not isinstance(location, int):
                raise ValueError(f"location not value {location}")
            if location == None:
                location = "create"
            if location in ["create", "modify", "access", ]:
                file_list = [os.path.join(dir_path, i) for i in file_list]
                if location == "create":
                    file_list.sort(
                        key=lambda x: FileBasicOperation.file_create_time(x) if x.endswith(endswith) else False)
                elif location == "modify":
                    file_list.sort(
                        key=lambda x: FileBasicOperation.file_modify_time(x) if x.endswith(endswith) else False)
                elif location == "access":
                    file_list.sort(
                        key=lambda x: FileBasicOperation.file_modify_time(x) if x.endswith(endswith) else False)
                return file_list
            elif location == "right":
                file_list.sort(key=lambda x: int(x[:-length].split(split)[-1]) if x.endswith(endswith) else False)
            elif location == "left":
                file_list.sort(key=lambda x: int(x[:-length].split(split)[0]) if x.endswith(endswith) else False)
            elif isinstance(location, int):
                file_list.sort(key=lambda x: int(x[:-length].split(split)[location]) if x.endswith(endswith) else False)
        else:
            file_list.sort(key=lambda x: x if x.endswith(endswith) else False)
        file_list = [os.path.join(dir_path, i) for i in file_list]
        return file_list

    # </editor-fold>
    # <editor-fold desc="文件夹数字排序">
    @staticmethod
    def dir_num_sort(dir_path, absolute=True):
        """

        :param dir_path: 文件位置
        :param absolute: 返回绝对位置
        :return:
        """
        rootpath = os.listdir(dir_path)
        rootpath = [dir_path for dir_path in rootpath if '.' not in dir_path]
        rootpath.sort(key=lambda x: int(x))
        if absolute:
            rootpath = [dir_path + '\\' + i for i in rootpath]
        return rootpath

    # </editor-fold>
    # <editor-fold desc="文件移动到另外一个问价夹内">
    @staticmethod
    def files_move(file, to_dir):
        """
        :param path:文件地址
        :param to_dir: 存储文件夹
        :return:
        """
        try:
            copy2(file, to_dir)
            # 将目标文件移动到目标文件夹里，
            # shutil.move(r'.\practice.txt', r'.\文件夹1/')
            # 将目标文件移动到目标文件夹里的同时，能够对其进行重命名
            # shutil.move(r'.\practice.txt', r'.\文件夹1/new.txt')
            # 如果我们需要移动某个或某些文件到新的文件夹，并且需重命名文件，
            # 则我们并不需要用 os.rename 先命名文件再用
            # shutil.move 将其移动的指定文件夹，而是可以用 shutil.move 一步到位
            return True
        except Exception as e:
            return False

    # </editor-fold>
    # <editor-fold desc="文件批量改名">
    @staticmethod
    def file_many_rename(dir_path, endswith):
        """

        :param dir_path:目录地址
        :param endswith: 文件结尾
        :return:
        """
        lis = list()
        for curDir, dirs, files in os.walk(dir_path):
            lis.append([os.path.join(curDir, file) for file in files if
                        file.endswith(endswith)
                        ]
                       )
        res = (x for j in lis for x in j)
        lisdir = list(res)
        fileNum = len(str(len(lisdir)))
        formatting_new = '%0' + str(fileNum) + 'd'
        for index, i in enumerate(lisdir):
            to_pa = i
            new = formatting_new % (index + 1)
            path = '\\'.join(i.split('\\')[:-1])
            file_name: str = i.split('\\')[-1]
            file_before = file_name.split('.')[0]
            if file_before.strip('0').isdigit():
                new_i = new + '.' + endswith
                new_path = os.path.join(path, new_i)
            elif file_before[:fileNum].strip('0').isdigit():
                new_path = new + file_before[fileNum:] + '.' + endswith
            else:
                new_path = new + file_before + '.' + endswith
            os.renames(to_pa, new_path)

    # </editor-fold>
    # <editor-fold desc="文件匹配">
    def glob(self, re_, root_dir, dir_fd=None, recursive=False):
        """

        :param endswith:
        :param root_dir:
        :param dir_fd:
        :param recursive:
        :return:
        glob 函数支持三种格式的语法：
* 匹配单个或多个字符
? 匹配任意单个字符
[] 匹配指定范围内的字符，如：[0-9]匹配数字。
        """
        # glob.glob("*.ipynb")
        # glob.glob("../09*/*.ipynb")
        # glob.glob("../[0-9]*")#匹配数字开头的文件夹名：
        # glob.glob("*.ipynb")
        return glob.glob(re_, root_dir=root_dir, dir_fd=dir_fd, recursive=recursive)

    # </editor-fold>
    def environment(self):
        """ 判断什么环境下
        windows 1
        linux 2
        mac 3
        """
        # 查看系统类型
        platform_ = platform.system()
        platformenv = 1  # 默认0 如果读不到,默认是在windows 环境下
        if platform_ == "Windows":
            platformenv = 1  # windows 1
        elif platform_ == "Linux":
            platformenv = 2  # linux 2
        elif platform_ == "Mac":
            platformenv = 3  # mac 3
        return platformenv

    def path(self, folderPath):
        env = self.environment()
        if folderPath is None:
            raise self.ConstCaseError("请传入文件夹路径,  项目/xxx文件夹  ")
        if (env == 1):
            folderPath = folderPath.replace("/", "\\")
            # print(pathUtil+folderPath)
            isok, msg = self.makedirs(folderPath)
            if not isok and msg != "":
                print("创建文件夹失败,请检查是否有此文件夹  ===>", folderPath)
            return folderPath
        if (env == 2):
            folderPath = folderPath.replace("/", "/")
            isok, msg = self.makedirs(folderPath)
            if not isok and msg != "":
                print("创建文件夹失败,请检查是否有此文件夹  ===>", folderPath)
            return folderPath
        if (env == 3):
            folderPath = folderPath.replace("/", "/")
            isok, msg = self.makedirs(folderPath)
            if not isok and msg != "":
                print("创建文件夹失败,请检查是否有此文件夹  ===>", folderPath)
            return folderPath
        self.makedirs(folderPath)
        return folderPath

    # <editor-fold desc="当前位置找到多层之后的父级">
    def path_layer(self, n, current_path=None):
        if not current_path:
            current_path = os.getcwd()
        for i in range(n):
            current_path = os.path.dirname(current_path)
        return current_path
    # </editor-fold>



"""

@Time : 2023/4/9 11:52 
@Author : skyoceanchen
@TEL: 18916403796
@项目：JYairports
@File : img_operations.by
@PRODUCT_NAME :PyCharm
"""
import base64
import qrcode


class ImageOperation(object):
    @staticmethod
    def base64_to_image(base64_data, img_file, ):
        with open(img_file, "wb") as f:
            f.write(base64_data)

    @staticmethod
    def image_to_base64(img_file, ):
        with open(img_file, "rb") as f:
            base64_data = base64.b64encode(f.read())
        return base64_data

    @staticmethod
    def base64_to_base64(base64_data):
        return base64_data

    @staticmethod
    def create_qrcode(text, file_path):
        # text = r"http://139.224.52.79:8089/media/report\\embed_report\\1648622788.pdf"  # 设置URL必须添加http://
        img = qrcode.make(text)
        img.save(file_path)  # 保存图片至本地目录，可以设定路径



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author:skyoceanchen
# project_name:myself_exercise
# py_name :保存excel
# software: PyCharm
# datetime:2021/6/26 15:27
import xlwt


# <editor-fold desc="保存excel--xlwt-xls">
class SaveExcelDateXLS(object):
    def __init__(self, file_name=None):
        """
        :param file_name:文件名称
        :param sheet_name: 表名
        :param header_list: 表头
        :param content_list: 内容 二进制
        """
        self.file_name = file_name
        # # 创建一个workbook 设置编码
        # workbook = xlwt.Workbook(encoding = 'utf-8')
        self.book = xlwt.Workbook(
            encoding='utf-8',
            # encoding='ascii',
            # style_compression=0
        )

    def add_sheet(self, sheet_name=None):
        self.sheet = self.book.add_sheet(sheet_name, cell_overwrite_ok=True)

    def header_table(self, header_list):
        for index, header in enumerate(header_list):
            self.sheet.write(0, index, header)

    def content(self, content_list):
        rows = len(content_list)
        # cols = len(content_list[0])
        old_rows = len(self.sheet.rows)
        for i in range(rows):
            for j in range(len(content_list[i])):
                self.sheet.write(i + old_rows, j, content_list[i][j])
        # print(111111111, self.sheet.rows,len(self.sheet.rows))
        # print(22222222222, self.sheet.col)

    def save(self):
        self.book.save(self.file_name)
# xls = SaveExcelDateXLS(file_name="1.xls")
# xls.add_sheet('wokk')
# xls.header_table([1, 2, 3, 4, 5])
# xls.header_table([1, 2, 3, 4, 5, 6, 7, 8, 9, 0])
# xls.header_table([3, 4, 52, 3, 4, 5])
# xls.content([[1, 2, 3, 4, 5], [5, 3, 2, "发射点发射点发射点发合法的手续和", 2], [4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 1, 2, 3]])
# xls.content([[1, 2, 3, 44, 5, 566, 76, 7, 1, 88, 8, 89, 9, 9, 1, 2, 3, 4, 5], [5, 3, 2, 1, 2],
#              [4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 1, 2, 3]])
#
# xls.save()
# </editor-fold>



# -*- coding: utf-8 -*-
"""
@Time : 2023/8/10 14:54 
@Author : skyoceanchen
@TEL: 18916403796
@项目：文件使用
@File : file_transform.by
@PRODUCT_NAME :PyCharm
"""
import os
import subprocess

import docx
import pdfkit
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfgen import canvas


# <editor-fold desc="HTML-转PDF">
class PdfKitOperation(object):
    def __init__(self, wkhtmltopdf_path):
        """
        下载：https://wkhtmltopdf.org/
        https://wkhtmltopdf.org/downloads.html
        :param wkhtmltopdf_path:wkhtmltopdf.exe的绝对路径
        """
        self.config = pdfkit.configuration(
            wkhtmltopdf=wkhtmltopdf_path)
        self.set_options()

    def set_options(self, orientation='Landscape'):
        """
        options = {
                # 'header-html': 'http://localhost:8080/static/data/pdfHeader.html',
                # # 设置页眉数据，作为页眉的html页面必须有<!DOCTYPE html>
                # 'header-spacing': '3',  # 设置页眉与正文之间的距离，单位是毫米
                # 'header-right': 'Quality Report',  # 设置页眉右侧数据
                # 'header-font-size': 10,  # 设置页眉字体大小
                # 'footer-font-size': 10,  # 设置页脚字体大小
                'footer-center': '[page]/[topage]页',  # 设置页码
                # 'margin-top': '0.75in',
                # 'margin-right': '0.75in',
                # 'margin-bottom': '0.5in',
                # 'margin-left': '0.75in',
                # 'encoding': "UTF-8",
                # # 'no-outline': None, #为None时表示确定，则不生成目录
                # 'header-line': None,  # 为None时表示确定，生成页眉下的线
                'orientation': 'Landscape',  # 横向
                "enable-local-file-access": True  # 打开本地文件访问权限
            }
        :param options:
        :return:
        """
        options = {}
        options['page-size'] = 'A4'
        options['footer-center'] = '[page]/[topage]页'
        if orientation:
            options['orientation'] = 'Landscape'
        options['enable-local-file-access'] = True
        self.options = {}

    def from_file(self, input_path, output_path):
        pdfkit.from_file(input_path, output_path, configuration=self.config, options=self.options)

    def from_url(self, url, output_path):

        pdfkit.from_url(url, output_path, configuration=self.config, options=self.options)

    def form_string(self, data, output_path):
        pdfkit.from_string(data, output_path, configuration=self.config, options=self.options)

    # 变化
    def water_report(self, output_path, templates_path, dic={}):
        with open(templates_path, 'r', encoding='utf-8') as f:
            data = f.read()
        # lis = ['provide_people', "tel", 'airport_code', 'assessment_datetime', 'track_code', 'status_code', 'degree',
        #        'degree_deep', 'pavement_status', ]
        # lis_checkd = ['control_tower', 'control_tower_other']
        for i in dic.keys():
            data = data.replace(i, dic.get(i, ''))
        # control_tower = dic.get('control_tower', False)
        # control_tower_other = dic.get('control_tower_other', False)
        # if control_tower:
        #     data = data.replace(
        #         """<input class="input_agreement_protocol control_tower"  type="checkbox"/>""",
        #         """<input class="input_agreement_protocol control_tower" checked type="checkbox"/>""")
        # if control_tower_other:
        #     data = data.replace(
        #         """<input class="input_agreement_protocol control_tower_other" type="checkbox"/>""",
        #         """<input class="input_agreement_protocol control_tower_other" checked type="checkbox"/>""")
        self.form_string(data, output_path)


# </editor-fold>
class PandocOperations(object):
    def __init__(self, pandoc_path):
        """
        下载： https://www.pandoc.org/installing.html
        :param pandoc_path: pandoc.exe 绝对路径
        """
        self.pandoc_path = pandoc_path

    def md_pdf(self, md_path, pdf_path):
        system = f'{self.pandoc_path}  {md_path} -o {pdf_path}'
        print(system)
        os.system(system)

    def md_doc(self, md_path, pdf_path):
        system = f'{self.pandoc_path}  {md_path} -o {pdf_path}'
        print(system)
        os.system(system)

    def convert_docx_to_pdf(self, docx_file, pdf_file):
        command = f"{self.pandoc_path} {docx_file} -o {pdf_file} --pdf-engine=xelatex"
        subprocess.run(command, shell=True)


from docx2pdf import convert


class DocxToPDFOperations(object):
    def docx_to_pdf(self, input_path, output_path):
        """
        :param input_path: test.docx
        :param output_path: test.pdf
        :return:
        """
        convert("D:\Program\Anaconda3\envs\激光断面仪标定报告.docx", "D:\Program\Anaconda3\envs\激光断面仪标定报告.pdf")
        convert(input_path, output_path)


def convert_docx_to_pdf_2(docx_file, pdf_file, ttf_path):
    """

    :param docx_file:
    :param pdf_file:
    :param ttf_path: r'F:\ContentPro\PythonPro\automaticoffice\字体\youyuan.TTF'
    :return:
    """
    doc = docx.Document(docx_file)
    pdf = canvas.Canvas(pdf_file)
    # 加载字体
    pdfmetrics.registerFont(TTFont('Arial', ttf_path))
    for para in doc.paragraphs:
        pdf.setFont("Arial", 12)
        pdf.drawString(100, 700, para.text)
        pdf.showPage()

    pdf.save()

# 用法示例
# convert_docx_to_pdf(md_path, pdf_path)



# -*- coding: utf-8 -*-
"""
@Time : 2023/8/16 21:13 
@Author : skyoceanchen
@TEL: 18916403796
@项目：文件使用
@File : pdf_operation.by
@PRODUCT_NAME :PyCharm
"""
import pdfplumber  # pip install pdfplumber


class PdfOperation(object):
    def __init__(self, path):
        self.path = path

    def read_pdf(self):
        pdf = pdfplumber.open(self.path)
        text_list = []
        # 解析pdf全部内容
        for i in pdf.pages:
            text = i.extract_text().replace("\u3000", "").replace("\xa0", "").replace("—", "").replace("(cid:122)", '')
            # str1 += i.extract_text().replace('', '')
            text_list.append(text)
        text = ''.join(text_list)
        return text





#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author  :hstking
# E-mail  :hstking@hotmail.com
# Ctime   :2015/09/15
# Mtime   :
# Version :

import getpass
import logging
import os
import sys
from logging.handlers import TimedRotatingFileHandler


#### 定义MyLog类
class MyLog(object):
    #### 类MyLog的构造函数
    def __init__(self):
        # 获取用户名称
        self.user = getpass.getuser()
        # log记录表明用户
        self.logger = logging.getLogger(self.user)
        # log 等级
        self.logger.setLevel(logging.DEBUG)
        ####  日志文件名
        self.logFile = sys.argv[0][0:-3] + '.log'
        # log文件格式
        self.formatter = logging.Formatter('%(asctime)-12s %(levelname)-8s %(name)-10s %(message)-12s\r\n')
        ####  日志输出到日志文件内
        self.logHand = logging.FileHandler(self.logFile, encoding='utf8')
        self.logHand.setFormatter(self.formatter)
        self.logHand.setLevel(logging.DEBUG)
        # 日志显示到屏幕
        self.logHandSt = logging.StreamHandler()
        self.logHandSt.setFormatter(self.formatter)
        self.logHandSt.setLevel(logging.DEBUG)

        self.logger.addHandler(self.logHand)
        self.logger.addHandler(self.logHandSt)

    ####  日志的5个级别对应以下的5个函数
    def debug(self, msg):
        self.logger.debug(msg)

    def info(self, msg):
        self.logger.info(msg)

    def warn(self, msg):
        self.logger.warning(msg)

    def error(self, msg):
        self.logger.error(msg)

    def critical(self, msg):
        self.logger.critical(msg)


# 日志级别
CRITICAL = 50
FATAL = CRITICAL
ERROR = 40
WARNING = 30
WARN = WARNING
INFO = 20
DEBUG = 10
NOTSET = 0

CURRENT_PATH = os.path.dirname(os.path.abspath(__file__))
ROOT_PATH = os.path.join(CURRENT_PATH, os.pardir)
LOG_PATH = os.path.join(ROOT_PATH, 'log')
if not os.path.exists(LOG_PATH):
    os.makedirs(LOG_PATH)


class LogHandler(logging.Logger):
    """
    LogHandler
    """

    def __init__(self, name, level=DEBUG, stream=True, file=True):
        self.name = name
        self.level = level
        logging.Logger.__init__(self, self.name, level=level)
        if stream:
            self.__setStreamHandler__()
        if file:
            self.__setFileHandler__()

    def __setFileHandler__(self, level=None):
        """
        set file handler
        :param level:
        :return:
        """
        file_name = os.path.join(LOG_PATH, '{name}.log'.format(name=self.name))
        # 设置日志回滚, 保存在log目录, 一天保存一个文件, 保留15天
        file_handler = TimedRotatingFileHandler(filename=file_name, when='D', interval=1, backupCount=15)
        file_handler.suffix = '%Y%m%d.log'
        if not level:
            file_handler.setLevel(self.level)
        else:
            file_handler.setLevel(level)
        formatter = logging.Formatter('%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s')

        file_handler.setFormatter(formatter)
        self.file_handler = file_handler
        self.addHandler(file_handler)

    def __setStreamHandler__(self, level=None):
        """
        set stream handler
        :param level:
        :return:
        """
        stream_handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s')
        stream_handler.setFormatter(formatter)
        if not level:
            stream_handler.setLevel(self.level)
        else:
            stream_handler.setLevel(level)
        self.addHandler(stream_handler)

    def resetName(self, name):
        """
        reset name
        :param name:
        :return:
        """
        self.name = name
        self.removeHandler(self.file_handler)
        self.__setFileHandler__()



# -*- coding: utf-8 -*-
"""
@Time : 2023/8/16 21:09 
@Author : skyoceanchen
@TEL: 18916403796
@项目：文件使用
@File : docx_operation.by
@PRODUCT_NAME :PyCharm
"""
import datetime

import docx  # pip install python-docx
from docx import Document  # 用于新建文档
from docx.enum.style import WD_STYLE_TYPE
from docx.enum.table import WD_TABLE_ALIGNMENT  # 对齐库
from docx.oxml.ns import qn  # 导入中文格式库
from docx.shared import Cm  # docx文档页边距
from docx.shared import Pt  # 导入字号，颜色库，注意大小写
from docx.shared import RGBColor  # 颜色库，注意大小写


class DocxOperation(object):
    def __init__(self, path):
        self.path = path

    def read_docx(self):
        doc = docx.Document(self.path)  # 绝对路径
        # 读取表格外全部内容
        text_list = []
        for i in doc.paragraphs:
            text = i.text.replace("—", "")
            if text:
                text_list.append(text)
        text = ''.join(text_list)
        return text


class DocxOperationCreate(object):
    def __init__(self):
        self.document = Document()

    def global_config(self):
        # 向document文档写入数据的过程
        # 设置全局样式
        self.document.styles['Normal'].font.name = 'Times New Roman'  # 英文和数字的字体设置；也可以直接设置仿宋_GB2312等中文格式
        self.document.styles['Normal']._element.rPr.rFonts.set(qn('w:eastAsia'),
                                                               '仿宋_GB2312')  # 指定全文非标题（即正文）中文字体进行格式设置。仿宋_GB2312
        self.document.styles['Normal'].font.size = Pt(16)  # 字号    14：四号；16：三号；22号：二号
        self.document.styles['Normal'].font.color.rgb = RGBColor(0, 0, 0)  # 颜色黑色
        self.document.styles['Normal'].paragraph_format.line_spacing = Pt(30)  # 若=1则行距为1行；若=docx.shared.Pt(30)则行距为30磅
        self.document.styles['Normal'].paragraph_format.space_after = Pt(0)  # 段后距离0磅
        self.document.styles['Normal'].paragraph_format.space_before = Pt(0)  # 段前距离0磅
        self.document.sections[0].top_margin = Cm(3.0)  # 新建一个空白文档，那么默认是只有一个节，如果这里没有[0],会报错
        self.document.sections[0].bottom_margin = Cm(3.0)
        self.document.sections[0].left_margin = Cm(2.6)
        self.document.sections[0].right_margin = Cm(2.6)
        # print('上',self.document.sections[0].top_margin.cm,'下',self.document.sections[0].bottom_margin.cm,'左',self.document.sections[0].left_margin.cm,'右',self.document.sections[0].right_margin.cm)
        # 也可以用下面这个表达式，注意如果后面不加.cm，输出的数字就不是cm
        print('上%.2fcm,下%.2fcm,左%.2fcm,右%.2fcm.' % (
            self.document.sections[0].top_margin, self.document.sections[0].bottom_margin.cm,
            self.document.sections[0].left_margin.cm,
            self.document.sections[0].right_margin.cm))

    def add_heading(self, text, level=0, alignment=1):
        """
        :param title:
        :param level: level 标题等级， 0-9
        :param alignment: 1 为居中 0 左 2右 0，1，2，3，4，5，7，8，9
        :return:
        """
        heading = self.document.add_heading(text, level)
        heading.alignment = alignment
        print(heading, dir(heading))
        print(heading.style, dir(heading.style))
        # self.font(heading.style, bold=True, italic=True, underline=True, strike=True, shadow=True, size=30)
        # heading.style.font.size = Pt(100)
        heading.style.font.name = "微软雅黑"
        print(heading._element)
        heading.style._element.rPr.rFonts.set(qn('w:eastAsia'), '黑体')  # 中文字体
        # heading.style._element.rPr.rFonts.set(qn('w:eastAsia'), "微软雅黑")  # 中文字体

    # _element
    # r.rPr.rFonts.set(qn('w:eastAsia'), '黑体')
    def font_run(self,
                 obj, bold=False, italic=False, underline=False, strike=False, shadow=False, size=None,
                 rgb=(0, 0, 0), font_name=None, alignment=None
                 ):
        """

        :param obj:  paragraph.add_run(title='HWD 标定报告', )
        :param bold:
        :param italic:
        :param underline:
        :param strike:
        :param shadow:
        :param size:
        :param rgb:
        :param font_name:
        :param alignment:
        :return:
        """
        obj.font.bold = bold  # 加粗
        obj.font.italic = italic  # 斜体
        obj.font.underline = underline  # 下划线
        obj.font.strike = strike  # 删除线
        obj.font.shadow = shadow  # 阴影
        obj.font.color.rgb = RGBColor(*rgb)
        if size:
            obj.font.size = Pt(size)  # 字体大小
        if font_name:
            obj.font.name = font_name
            obj._element.rPr.rFonts.set(qn('w:eastAsia'), font_name)  # 中文字体
        if alignment or alignment == 0:
            obj._parent.alignment = alignment
        return obj

    def font_paragraph(self, paragraph, bold=False, italic=False, underline=False, strike=False, shadow=False,
                       size=None, rgb=(0, 0, 0), font_name=None, alignment=None,
                       line_spacing=0, space_after=0, space_before=0, first_line_indent=0
                       ):
        obj = paragraph.style
        obj.font.bold = bold  # 加粗
        obj.font.italic = italic  # 斜体
        obj.font.underline = underline  # 下划线
        obj.font.strike = strike  # 删除线
        obj.font.shadow = shadow  # 阴影
        obj.font.color.rgb = RGBColor(*rgb)
        if size:
            obj.font.size = Pt(size)  # 字体大小
        if obj.font.size:
            size = obj.font.size
        else:
            size = 11
        if font_name:
            obj.font.name = font_name
            obj._element.rPr.rFonts.set(qn('w:eastAsia'), font_name)  # 中文字体
        if alignment or alignment == 0:
            paragraph.alignment = alignment
        if line_spacing or line_spacing == 0:
            paragraph.paragraph_format.line_spacing = Pt(line_spacing)  # 若=1则行距为1行；若=docx.shared.Pt(30)则行距为30磅
        if space_after or space_after == 0:
            paragraph.paragraph_format.space_after = Pt(space_after)  # 段后距离0磅
        if space_before or space_before == 0:
            paragraph.paragraph_format.space_before = Pt(space_before)  # 段前距离0磅
        if first_line_indent:
            paragraph.paragraph_format.first_line_indent = Pt(size * first_line_indent)  # 2 # 设置段落的缩进量
        return paragraph

    def add_paragraph(self, conent=""):
        paragraph = self.document.add_paragraph(conent)
        return paragraph

    def add_run(self, paragraph=None, conent=""):
        if paragraph:
            run = paragraph.add_run(conent)
        else:
            run = self.document.add_paragraph().add_run(conent)
        return run

    def add_table(self, rows, cols):
        table = self.document.add_table(rows=rows, cols=cols)
        return table

    def table_merge(self, table, start, end):
        """
        :param table:
        :param start: 开始坐标
        :param end: 结束坐标
        :return:
        table.cell(0, 0).merge(table.cell(0, 1))
        """
        table.cell(*start).merge(table.cell(*end))
        return table
        #

    def table_content(self, table, content):
        """

        :param table:
        :param content:
        :return:
        """
        # 循环添加
        for i, row in enumerate(table.rows):

            for j, cell in enumerate(row.cells):
                cell.text = content[i][j]
                # 获取单元格中的段落对象
                # paragraph = cell.paragraphs[0]
                # # paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
                # # 这里的run可以设置一些属性
                # run = paragraph.add_run(content[i][j])
                # # if i == 2:
                # #     run.font.color.rgb = RGBColor(0, 255, 0)

    def font_table(self, table,
                   bold=False, italic=False, underline=False, strike=False, shadow=False, size=None,
                   rgb=(0, 0, 0), font_name=None, alignment=None
                   ):
        """

        :param table:  document.add_table(rows=rows, cols=cols)
        :param bold:
        :param italic:
        :param underline:
        :param strike:
        :param shadow:
        :param size:
        :param rgb:
        :param font_name:
        :param alignment:
        :return:
        """
        obj = table.style
        obj.font.bold = bold  # 加粗
        obj.font.italic = italic  # 斜体
        obj.font.underline = underline  # 下划线
        obj.font.strike = strike  # 删除线
        obj.font.shadow = shadow  # 阴影
        obj.font.color.rgb = RGBColor(*rgb)
        if size:
            obj.font.size = Pt(size)  # 字体大小
        if font_name:
            obj.font.name = font_name
            obj._element.rPr.rFonts.set(qn('w:eastAsia'), font_name)  # 中文字体
        if alignment:
            table.style.paragraph_format.alignment = alignment

    def table_div_style(self, table_style_name='mystyle', bold=False, italic=False, underline=False, strike=False,
                        shadow=False, size=None, rgb=(0, 0, 0), font_name=None, alignment=None):
        """

        :param table_style_name:
        :param bold:
        :param font_name:
        :param size:
        :param rgb:
        :param alignment:
        :return:

        用法：
         # 创建一个带有三列和四行的表格
        # table = document.add_table(rows=4, cols=3,style='mystyle')
        table = document.add_table(rows=4, cols=3)
        # 设置表格样式
        table.style = obj
        """
        # 定义样式
        table_style_name = table_style_name
        obj = self.document.styles.add_style(table_style_name, WD_STYLE_TYPE.TABLE)
        obj.font.bold = bold  # 加粗
        obj.font.italic = italic  # 斜体
        obj.font.underline = underline  # 下划线
        obj.font.strike = strike  # 删除线
        obj.font.shadow = shadow  # 阴影
        obj.font.color.rgb = RGBColor(*rgb)
        if alignment or alignment == 0:
            obj.paragraph_format.alignment = alignment
        if size:
            obj.font.size = Pt(size)  # 字体大小
        if font_name:
            obj.font.name = font_name
            obj._element.rPr.rFonts.set(qn('w:eastAsia'), font_name)  # 中文字体
        return obj

    def table_global_config(self, table):

        table.alignment = WD_TABLE_ALIGNMENT.CENTER  # 对表格进行对齐
        # Turn off autofit  固定列宽
        table.autofit = False

    # 换行
    def add_break(self):
        self.font_paragraph(self.document.add_paragraph("\t"))

    # 分页
    def add_page_break(self):
        self.document.add_page_break()


# 案例
def LoaderReport(date=datetime.datetime.now().strftime('%Y年%m月%d日'),
                 area="同济大学嘉定校区智能网联交通测评基地(北区)",
                 creator="佚名",
                 creator_sign="",
                 test_by="同科",
                 brand="熙赢测控",
                 version="SHN-SF",
                 number="SHN-SF（无）",
                 operators="匿名"
                 ):
    """
    标定日期，标定场地，标定人员，标定人员签字，测试方，品牌，型号，编号，操作人员
    :param date:
    :param area:
    :param creator:
    :param creator_sign:
    :param test_by:
    :param brand:
    :param version:
    :param number:
    :param operators:
    :return:
    """
    doc = DocxOperationCreate()
    run = doc.add_run(conent='激光断面仪标定报告', )
    doc.font_run(run, bold=True, alignment=1, font_name="微软雅黑")
    doc.add_break()
    doc.add_break()
    doc.add_break()

    paragraph = doc.add_paragraph("\t标定日期:")
    doc.font_paragraph(paragraph, bold=True, font_name="微软雅黑")
    run = doc.add_run(paragraph, f"\t{date}")
    doc.font_run(run, bold=False)

    paragraph = doc.add_paragraph("\t标定场地:")
    doc.font_paragraph(paragraph, bold=True, font_name="微软雅黑")
    run = doc.add_run(paragraph, f"\t{area}")
    doc.font_run(run, bold=False)

    paragraph = doc.add_paragraph("\t标定人员:")
    doc.font_paragraph(paragraph, bold=True, font_name="微软雅黑")
    run = doc.add_run(paragraph, f"\t{creator}")
    doc.font_run(run, bold=False)

    paragraph = doc.add_paragraph("\t标定人员签名")
    doc.font_paragraph(paragraph, bold=True, font_name="微软雅黑")
    run = doc.add_run(paragraph, f"\t\t\t\t{creator_sign}")
    doc.font_run(run, bold=False, underline=True)
    doc.add_break()
    doc.add_break()
    doc.add_break()
    lis = [
        ["激光断面仪测试方:", test_by],
        ["激光断面仪品牌:", brand],
        ["激光断面仪型号:", version],
        ["激光断面仪编号:", number],
        ["激光断面仪操作人员:", operators],
    ]
    for li in lis:
        paragraph = doc.add_paragraph(f"\t{li[0]}")
        doc.font_paragraph(paragraph, bold=True, font_name="微软雅黑")
        run = doc.add_run(paragraph, f"\t{li[1]}")
        doc.font_run(run, bold=False, )
    doc.add_break()
    doc.add_break()
    doc.add_break()
    paragraph = doc.add_paragraph(f"\t激光断面仪随机误差")
    doc.font_paragraph(paragraph, bold=True, font_name="微软雅黑")

    table1_content = [
        [
            ["" for i in range(11)] + ['IRI 0~2m/km 随机误差检测结果'],
            ['测试', ] + [str(i) for i in range(1, 11)] + ['变异系数'],
            ['1', ] + [str(i) for i in range(1, 11)] + ['变异系数'],
            ['2', ] + [str(i) for i in range(1, 11)] + ['变异系数'],
        ],

    ]
    for table_content in table1_content:
        table = doc.add_table(len(table_content), len(table_content[0]))
        table.style = "Table Grid"  # Colorful List
        doc.font_table(table, alignment=1)
        print(len(table_content[0]))
        doc.table_merge(table, start=(0, 0), end=(0, len(table_content[0]) - 1))
        doc.table_content(table, table_content)
        doc.add_break()
    paragraph = doc.add_paragraph(f"\t标定模型")
    doc.font_paragraph(paragraph, bold=True, font_name="微软雅黑")
    table2_content = [
        [
            ['', '激光断面仪（单激光）'],
            ['安装位置', '测试车右侧'],
            ['系统误差', ''],
            ['标定方程', '摩擦系数真值 = 1.0214 * 摩擦系数测试值 - 0.0115'],
            ['决定系数（R2）', '003'],
            ['误差来源', '003\n005']
        ],
    ]
    for table_content in table2_content:
        table = doc.add_table(len(table_content), len(table_content[0]))
        table.style = "Table Grid"  # Colorful List
        doc.font_table(table, alignment=1)
        doc.table_merge(table, start=(0, 0), end=(0, len(table_content[0]) - 1))
        doc.table_content(table, table_content)
        doc.add_break()

    paragraph = doc.add_paragraph(f"标定结论")
    doc.font_paragraph(paragraph, bold=True, first_line_indent=2, font_name="微软雅黑")
    paragraph = doc.add_paragraph()
    paragraph = doc.font_paragraph(paragraph=paragraph, first_line_indent=2, font_name="微软雅黑")

    run = doc.add_run(paragraph, conent=f"""测试方""")
    doc.font_run(run, bold=False, font_name="微软雅黑")
    run = doc.add_run(paragraph, conent=f""" {test_by} """)
    doc.font_run(run, bold=True, font_name="微软雅黑")
    run = doc.add_run(paragraph, conent=f"""来标定的激光断面仪品牌为""")
    doc.font_run(run, bold=False, font_name="微软雅黑")
    run = doc.add_run(paragraph, conent=f""" {brand} """)
    doc.font_run(run, bold=True, font_name="微软雅黑")
    run = doc.add_run(paragraph, conent=f"""型号为""")
    doc.font_run(run, bold=False, font_name="微软雅黑")
    run = doc.add_run(paragraph, conent=f""" {number},""")
    doc.font_run(run, bold=True, font_name="微软雅黑")
    run = doc.add_run(paragraph, conent=f"""接受机场道面适航安全测试设备标定管理平台的标定。""")
    doc.font_run(run, bold=False, font_name="微软雅黑")

    paragraph = doc.add_paragraph(conent=f"""标定结论和依据如下：""")
    doc.font_paragraph(paragraph, bold=False, first_line_indent=2, font_name="微软雅黑")
    lis = ["""（1）标定中 不通过 随机误差校验（变异系数 > 15%）""",
           """（2）标定结论为激光断面仪设备 不通过 本次标定。""",
           """（3）标定结论为激光断面仪设备 通过 本次标定。""",
           ]
    for li in lis:
        paragraph = doc.add_paragraph(conent=f"""{li}""")
        doc.font_paragraph(paragraph, bold=False, first_line_indent=2, font_name="微软雅黑")
    paragraph = doc.add_paragraph(conent=f"""根据标定数据和结果给出以下建议:""")
    doc.font_paragraph(paragraph, bold=False, first_line_indent=2, font_name="微软雅黑")
    lis2 = [
        """（1） 标定的激光断面仪存在随机误差，建议检查或更换安装支架及激光断面仪。""",
    ]
    for li in lis2:
        paragraph = doc.add_paragraph(conent=f"""{li}""")
        doc.font_paragraph(paragraph, bold=False, first_line_indent=2, font_name="微软雅黑")
    doc.document.save('激光断面仪标定报告.docx')

# if __name__ == '__main__':
#     LoaderReport()



# 获取项目路径
import os.path

import yaml


class YamlUtil:
    # 读取config.yml文件
    def read_config_file(self, one_node, two_node, comfig_yaml="config/config.yml"):
        with open(comfig_yaml, 'r', encoding='utf-8') as f:
            value = yaml.load(stream=f, Loader=yaml.FullLoader)
            return value[one_node][two_node]

    # 读yml
    def read_yaml(self, yaml_file):
        with open(yaml_file, encoding='utf-8') as f:
            value = yaml.load(stream=f, Loader=yaml.FullLoader)
        return value

    # yaml.yml文件
    def write_yaml(self, data, yaml_file):
        with open(yaml_file, encoding='utf-8', mode='a') as f:
            yaml.dump(data=data, stream=f, allow_unicode=True)

    # 读取extract.yml文件
    def read_yaml_file(self, node_name=None, yaml_file="extract.yml"):
        with open(yaml_file, encoding='utf-8') as f:
            value = yaml.load(stream=f, Loader=yaml.FullLoader)
            if node_name == None:
                return value

            elif node_name:
                return value[node_name]

    # 清空yaml_file文件
    def clear_yaml(self, yaml_file):
        with open(yaml_file, encoding='utf-8', mode='w') as f:
            f.truncate()

    def remove_yaml(self, yaml_file):
        os.remove(yaml_file)



# -*- coding: utf-8 -*-
"""
@Time : 2023/8/16 21:13 
@Author : skyoceanchen
@TEL: 18916403796
@项目：文件使用
@File : pdf_operation.by
@PRODUCT_NAME :PyCharm
"""
# https://pyfpdf.readthedocs.io/en/latest/reference/close/index.html
# https://pyfpdf.readthedocs.io/en/latest/reference/set_creator/index.html
# https://pyfpdf.readthedocs.io/en/latest/
from fpdf import FPDF  # pip install fpdf==1.7.2


# 页面在A4纵向和度量单位是毫米。它可以通过以下方式明确指定：
# pdf = FPDF('P', 'mm', 'A4')
# 它可以使用横向（L），其他页面的格式（如Letter和 Legal），测量单元（pt，cm，in）
# pdf = FPDF()
# pdf.add_page()  # 添加一页
# pdf.set_font('Arial', 'B', 16)  # 设置字体 Arial粗体16
# pdf.cell(40, 10, 'Hello World!')  # 设置位置，添加数据
# pdf.cell(40, 10, 'Hello World!', 1)
# pdf.cell(60, 10, 'Powered by FPDF.', 0, 1, 'C')
# pdf.output('tuto1.pdf', 'F')  # 保存文件
# pdf.close()  # 关闭
# pdf.footer()


# pdf.accept_page_break()
# pdf.set_auto_page_break()
class PDF(FPDF):
    def __init__(self, orientation, unit, format):
        super(PDF, self).__init__(orientation=orientation, unit=unit, format=format)

    # 获取当前页码
    def page_no(self):
        "获取当前页码"
        return self.page

    # 别名nb_页:为页面总数定义一个别名
    def alias_nb_pages(self, alias='{nb}'):
        return super().alias_nb_pages(alias=alias)

    # 错误信息。
    def error(self, msg=str):
        """
        如果发生致命错误，将自动调用此方法；它只需输出消息并停止执行。
        继承的类可能会重写它以自定义错误处理，但应该始终停止脚本，否则生成的文档可能无效。
        :param msg:错误信息。
        :return:
        """
        super().error(msg=msg)

    #   激活或取消页压缩
    def set_compression(self, compress: bool):
        """
        激活或取消页压缩。当激活时，每个页面的内部表示都会被压缩，这将导致结果文档的压缩比约为2。
        默认情况下，压缩是打开的。
        :param compress:布尔值，指示是否必须启用压缩。
        :return:
        """
        super().set_compression(compress=compress)

    def set_display_mode(self, zoom, layout='continuous'):
        """

        :param zoom:要用的变焦。它可以是以下字符串值之一：
                    fullpage：：在屏幕上显示整个页面
                    fullwidth使用窗口的最大宽度
                    real使用实际大小(相当于100%缩放)
                    default*使用查看器默认模式
                    或表示要使用的缩放因子的数字，作为百分比。
        :param layout:页面布局。可能的价值是：
                    single：：每次显示一页
                    continuous*连续显示页面
                    two*在两列中显示两页
                    default*使用查看器默认模式
                    默认值是continuous.
        :return:
        """
        # 描述
        """
        定义查看器显示文档的方式。
        可以设置缩放级别：页面可以完全显示在屏幕上，
        占据窗口的全部宽度，使用实际大小，通过特定的缩放因子缩放
        ，或者使用查看器默认设置(在AdobeReader的“首选项”菜单中配置)。
        也可以指定页面布局：一次单页、连续显示、两列或查看器默认设置。
        如果未调用此方法，则将缩放模式设置为全宽而布局设置为连续默认情况下。
        """
        super().set_display_mode(zoom=zoom, layout=layout)

    # 定义文档的作者
    def set_author(self, author: str):
        """
        :param author: 作者的名字
        :return:
        """
        super().set_author(author=author)

    #  创建者
    def set_creator(self, creator: str):
        """
        定义文档的创建者。这通常是生成PDF的应用程序的名称。
        :param creator:创建者
        :return:
        """
        super().set_creator(creator=creator)

    # 关键词:关键字与文档相关联，通常以“keyword 1 keyword 2.”形式出现。
    def set_keywords(self, keywords: str):
        """
        :param keywords:关键词：关键词列表。形式->"keyword 1 keyword 2."
        :return:
        """
        super().set_keywords(keywords=keywords)

    # 定义文档的主题。
    def set_subject(self, subject: str):
        """
        :param subject: 主题
        :return:
        """
        super().set_subject(subject=subject)
    #
    # # 定义文档的标题。
    # def set_title(self, title: str):
    #     """
    #     :param title:标题。
    #     :return:
    #     """
    #     super().set_title(title=title)


class MyPDF(PDF):
    def __init__(self, orientation='P', unit='mm', format='A4'):
        """这是类构造函数。它允许设置所有方法中使用的页面格式、方向和度量单位(字体大小除外)。"""
        """
        :param orientation:页面方向。可能的值是(不区分大小写的)：
                            P or Portrait-----P或肖像
                            L or Landscape-----L或景观
                            默认值是传递给构造函数的值。
        :param unit: 用户单位。可能的价值是：
                        pt: point————PT：点
                        mm: millimeter————毫米：毫米
                        cm: centimeter————厘米：厘米
                        in: inch————英寸
        :param format:用于页面的格式。它可以是下列值之一(不区分大小写)：
                        A3----A3
                        A4----A4
                        A5----A5
                        Letter----信件
                        Legal----合法
                        或包含宽度和高度的元组(以给定单位表示)。在纵向定位中，元组应按顺序排列(宽度, 高度)，
                        但在景观方向上，顺序应该是(高度, 宽度)。在任何一种情况下，第一个元组元素通常小于第二个元组元素。
                        默认值是传递给构造函数的值。
                        默认值是A4。 
        """
        super(PDF, self).__init__(orientation=orientation, unit=unit, format=format)
        """
        使用自定义的100x150毫米页格式的示例：
        pdf = FPDF('P', 'mm', (100, 150))
        """

    def open(self):
        """
        此方法开始生成PDF文档。没有必要显式地调用它，因为添加页自动完成。
注意：此方法不会创建任何页面。
        :return:
        """
        super().open()

    # 在您自己的继承类中实现的头文件
    def header(self):
        """
        此方法用于呈现页面标题。
        它自动被添加页而不应由应用程序直接调用。
        FPDF中的实现是空的，所以如果需要特定的处理，
        就必须对它进行子类化并重写该方法。
        :return:
        """
        # Select Arial bold 15
        self.set_font('Arial', 'B', 15)
        # Move to the right
        self.cell(80)
        # Framed title
        # self.cell(30, 10, 'Title', 1, 0, 'C')
        # Line break
        self.ln(20)

    # 页面页脚
    def footer(self):
        """
        此方法用于呈现页面页脚。它由add_page自动调用并关闭，并且不应由应用程序直接调用。
        FPDF中的实现为空，因此如果要进行特定处理，则必须对其进行子类化并重写该方法。
        :return:
        """
        # Do not print footer on first page
        # 不要在第一页打印页脚
        if self.page_no() != 1:
            # Go to 1.5 cm from bottom
            self.set_y(-15)
            # Select Arial italic 8
            self.set_font('Arial', 'I', 8)
            # Print centered page number
            self.cell(0, 10, 'Page %s' % self.page_no(), 0, 0, 'C')

    # 添加页
    def add_page(self, orientation='', format='', same=False):
        """
        :param orientation:页面方向。可能的值是(不区分大小写的)：
                            P or Portrait-----P或肖像
                            L or Landscape-----L或景观
                            默认值是传递给构造函数的值。
        :param format:用于页面的格式。它可以是下列值之一(不区分大小写)：
                        A3----A3
                        A4----A4
                        A5----A5
                        Letter----信件
                        Legal----合法
                        或包含宽度和高度的元组(以给定单位表示)。在纵向定位中，元组应按顺序排列(宽度, 高度)，
                        但在景观方向上，顺序应该是(高度, 宽度)。在任何一种情况下，第一个元组元素通常小于第二个元组元素。
                        默认值是传递给构造函数的值。
        :param same:如果页面必须与前一页相同，则为true。在这种情况下，其他参数将被忽略。
        :return:
        """
        # 描述
        """向文档添加新页。如果页已经存在，则页脚方法首先被调用以输出页脚。
        然后添加页面，根据左上角和上边框将当前位置设置为左上角，以及标头调用以显示标题。
        调用前设置的字体将自动还原。
        没有必要打电话设置字体同样，如果要继续使用相同的字体。颜色和线宽也保留下来。
        坐标系的起源在左上角，增加的纵坐标向下移动。
        """

        super().add_page(orientation=orientation)

    # 是否接受自动换页
    def accept_page_break(self):
        # <editor-fold desc="是否接受自动换页">
        """
        每当满足分页中断条件时，将调用此方法，并根据返回的值发出或不发出该中断。
        默认实现根据所选择的模式返回一个值。设置自动分页。此方法是自动调用的，应用程序不应直接调用该方法。
        :return:
        """
        return self.auto_page_break
        # </editor-fold>

    # 设置自动换页模式和触发页边距
    def set_auto_page_break(self, auto: bool, margin=0.0):
        # <editor-fold desc="设置自动换页模式和触发页边距">

        """
        启用或禁用自动分页模式。启用时，第二个参数是与定义触发限制的页面底部的距离。默认情况下，模式是开的，边距是2cm。
        :param auto:布尔值，指示模式是打开还是关闭。
        :param margin:与页面底部的距离。
        :return:
        """
        "设置自动换页模式和触发页边距"
        self.auto_page_break = auto
        self.b_margin = margin
        self.page_break_trigger = self.h - margin

        # </editor-fold>

    # 添加TrueType或Type1字体
    def add_font(self, family: str, style='', fname='', uni=False):
        # <editor-fold desc="添加TrueType或Type1字体">
        """
        :param pdf:
        :param family:字体系列。用作set_font的参考，例如：'dejavu'。
        :param style:字体样式。不推荐使用，仅为了向后兼容而进行维护。
        :param fname:字体文件名（例如'DejaVuSansCondensed.ttf'）。您可以指定完整路径；否则，将在FPDF_FONTPATH或中搜索文件SYSTEM_TTFONTS。
        :param uni:TTF Unicode标志（如果设置为True，则将启用TrueType字体子集嵌入，并且utf8默认情况下将文本视为文本）。
        :return:
        你一定不能调用add_font为标准PDF的Latin-1字体（快递，黑体，时代，符号，ZAPFDINGBATS）; 在这种情况下，请直接使用set_font。
        """
        super().add_font(family, style=style, fname=fname, uni=uni)
        """
        # Add a Unicode free font
        pdf.add_font('DejaVu', '', 'DejaVuSansCondensed.ttf', uni=True)
        # Add a Unicode system font (using full path)
        pdf.add_font('sysfont', '', r"c:\WINDOWS\Fonts\arial.ttf", uni=True)
        描述
        导入TrueType，OpenType或Type1字体并使其可用。
        警告：对于Type1和旧字体，必须首先使用该MakeFont实用程序生成字体定义文件。目前不推荐使用此功能，而支持TrueType Unicode字体（使用随附的ttfonts.py实用程序会自动处理其字体）。
        注意：字体源文件必须可访问。将在以下位置连续搜索它们（如果定义了这些常量）：
        FPDF_FONTPATH（默认情况下，font位于fpdf软件包目录中的文件夹）
        SYSTEM_TTFONTS（例如C:\WINDOWS\FONTS）
        如果找不到与所请求字体相对应的文件，则会引发运行时异常“未找到TTF字体文件：...”。
        有关更多信息，请参见Unicode支持页面。
        应该在使用set_font方法之前调用此方法，并且该字体将可用于整个文档。
        注意：由于字体处理会占用大量时间，因此会缓存一些数据。
        默认情况下，缓存文件是在同一文件夹中创建的。可以通过设置FPDF_CACHE_MODE常量来更改：
        0-（默认情况下），将缓存存储在字体文件所在的文件夹中
        1-禁用所有缓存
        2-将高速缓存文件存储在FPDF_CACHE_DIR具有神秘名称的目录中
        """

        # </editor-fold>

    # 选择一个字体;以点数表示的尺寸
    def set_font(self, family, style='', size=0):
        # <editor-fold desc="选择一个字体;以点数表示的尺寸">
        """
        :param family:
                字体家族。它可以是由添加字体或其中一个标准家庭(不区分大小写)：
                信使(固定宽度) ————Courier (fixed-width)
                Helvetica或Arial(同义词；无衬线) ————Helvetica or Arial (synonymous; sans serif)
                时间(衬线) ————Times (serif)
                符号(符号) ————Symbol (symbolic)
                ZapfDingbat(象征性) ————ZapfDingbats (symbolic)
                还可以传递空字符串。在这种情况下，保留目前的字体。

        :param size:字体大小(以点为单位)。默认值是当前大小。如果自文档开始以来没有指定任何大小，则所取值为12。
        :param style:字体风格。可能的值是(不区分大小写的)：
                    空字符串：普通字符串
                    B：粗体
                    I：斜体
                    U：划线
                    或者任何组合。默认值是规则的。粗体和斜体风格不适用于符号和ZapfDingbat。

        :return:
        """
        # 描述
        """
        设置用于打印字符串的字体。在打印文本或结果文档无效之前，必须至少调用一次此方法。
        字体可以是标准字体，也可以是通过添加字体方法。
        未指定默认编码，但所有文本写入方法只接受外部字体的Unicode和标准的一个字节编码。
        标准字体使用Latin-1默认情况下编码，但Windows编码cp1252(西欧)可与SET_doc选项(“core_fonts_coding”，编码)。
        可以在创建第一页之前调用该方法，并在页与页之间保留字体。
        如果您只想更改当前的字体大小，那么调用它就更简单了。设定字体大小.
        注字体度量文件必须是可访问的。他们先后在下列地点被搜查：
        由FPDF_FONTPATH常量定义的目录(如果定义了此常量)
        位于包含fpdf.py的目录中的字体目录(如果存在的话)
        通过include()示例访问的目录定义FPDF_FONTPATH(注意强制尾随斜杠)：Defined(“FPDF_FONTPATH”，“/HOME/www/字体/”)；Required(“fpdf.php”)；如果找不到与请求字体相对应的文件，则发出错误“无法包含字体度量文件”。
        """
        super().set_font(family=family, style=style, size=size)

        # </editor-fold>

    # 以点为单位设置字体大小
    def set_font_size(self, size):
        """
        定义当前字体的大小。
        :param size: 大小(以点数为单位)。
        :return:
        """
        super().set_font_size(size=size)

    # 创建一个新的内部链接
    def add_link(self):
        # <editor-fold desc="创建一个新的内部链接">
        """
        创建一个新的内部链接并返回其标识符。
        内部链接是指向文档中另一个位置的可点击区域。
        然后，可以将标识符传递给细胞, 写, 影象或链接。目标定义为设定链.
        :return:
        """
        return super().add_link()

        # </editor-fold>

    # 写入一个文本cell
    def cell(self, w, h=0, txt='', border=0, ln=0, align='', fill=False, link=''):
        # <editor-fold desc="写入一个文本cell">
        """
        打印带有可选边框、背景色和字符串的单元格(矩形区域)。单元格的左上角对应于当前位置.文本可以对齐或居中。
        调用后，当前位置移动到右侧或下一行。在文本上放一个链接是可能的。
        如果启用了自动分页，并且单元格超出了限制，则在输出之前会执行分页操作。
        :param w:宽度。如果为0，则该单元格扩展到右侧边缘。
        :param h:高度。默认值：0。
        :param txt:字符串。默认值：空字符串。
        :param border:指示是否必须在单元格周围绘制边框。该值可以是一个数字：
                        0：无边界
                        1：框架
                        或包含以下部分或全部字符的字符串(按任何顺序排列)：
                        L*左
                        T*顶部
                        R*对
                        B*底部
                        默认值：0。
        :param ln:指示调用后当前位置应移至何处。可能的价值是：
                    0：右边
                    1：到下一行的开头
                    2：下文
                    输入1等于将0放入并调用林就在这之后。默认值：0。
        :param align:允许对中或对齐文本。可能的价值是：
                        L或空字符串：左对齐(默认值)
                        C*中心
                        R*右对齐
        :param fill:指示是否必须绘制单元格背景(True)或透明(False)。默认值：false。
        :param link:返回的URL或标识符加链.add_link() 方法返回值
        :return:
        """
        super().cell(w=w, h=h, txt=txt, border=border, ln=ln, align=align, fill=fill, link=link)

        # </editor-fold>

    def multi_cell(self, w: float, h: float, txt: str = '', border=0, align: str = 'J', fill=0, split_only=False):
        """
        :param w:宽度。如果为0，则将其扩展到页的右边距。
        :param h:高度。
        :param txt:字符串
        :param border:指示是否必须围绕单元格块绘制边框。该值可以是一个数字：
                        0：无边界
                        1：框架
                        或包含以下部分或全部字符的字符串(按任何顺序排列)：
                        L*左
                        T*顶部
                        R*对
                        B*底部
                        默认值：0。
        :param align:设置文本对齐。可能的价值是：
            L*左对齐
            C*中心
            R*对齐
            J*理由(默认值)
        :param fill:指示是否必须绘制单元格背景(True)或透明(False)。默认值：false。
        :param split_only:
        :return:
        """
        # 描述
        """
        此方法允许打印带有换行符的文本。它们可以是自动的(当文本到达单元格的右边框时)或显式(通过\n)。
        输出尽可能多的单元格，其中一个在另一个下面。文本可以对齐、居中或对齐。该单元可以被框和背景油漆。
        """
        super().multi_cell(w=w, h=h, txt=txt, border=border, align=align, fill=fill, split_only=split_only)

    # 以流动模式输出文本
    def write(self, h: float, txt: str = '', link=''):
        # <editor-fold desc="以流动模式输出文本">
        """
        此方法从当前位置打印文本。当到达右边距(或满足\n字符)时，将发生行中断，文本从左侧边距继续。
        在方法退出时，当前位置就留在文本的末尾。在文本上放一个链接是可能的。
        :param h:线高。
        :param txt:字符串
        :param link:返回的URL或标识符加链. add_link() 方法返回值
        :return:
        """
        super().write(h=h, txt=txt, link=link)

        # </editor-fold>

    # 在页面上放一幅图片
    def image(self, name, x=None, y=None, w=0, h=0, type='', link=''):
        # <editor-fold desc="在页面上放一幅图片">
        """
        :param name:图像的路径或URL。
        :param x:左上角的横坐标。如果未指定或等于无，则使用当前的横坐标(版本1.7.1及以上).
        :param y:左上角的纵坐标。如果未指定或等于无，则使用当前纵坐标；此外，如果有必要，首先触发分页(在启用自动分页的情况下)，并且在调用之后，当前纵坐标被移动到图像的底部(版本1.7.1及以上).
        :param w:页面中图像的宽度。如果未指定或等于零，则自动计算。
        :param h:页面中图像的高度。如果未指定或等于零，则自动计算。
        :param type:图像格式可能的值是(不区分大小写的)：jpg、JPEG、PNG和GIF。如果未指定，则从文件扩展名推断类型。
        :param link:返回的URL或标识符加链. add_link() 方法返回值
        :return:
        """
        # 描述
        """
        放个图像。它将在页面上显示的大小可以通过不同的方式指定：
        显式宽度和高度(以用户单位表示)
        一个显式维数，另一个维数是自动计算的，以保持原来的比例。
        没有显式的维数，在这种情况下，图像被放置在72 dpi。
        支持的格式有JPEG、PNG和GIF。
        对于JPEG，所有口味都是允许的：
        灰阶
        真彩(24位)
        CMYK(32位)
        对于PNG，允许这样做：
        灰度等级最多为8位(256级)
        索引颜色
        真彩(24位)
        阿尔法通道(1.7及以上版本)
        但这一点没有得到支持：
        隔行
        对于GIF：在动画GIF的情况下，只使用第一个框架。
        如果定义了透明的颜色，就会考虑到它。
        可以从文件扩展名中显式指定或推断格式。
        可以在图像上添加链接。
        """
        super().image(name, x=x, y=y, w=w, h=h, type=type, link=link)

        # </editor-fold>

    # 在页面上放一个链接
    def link(self, x: float, y: float, w: float, h: float, link):
        # <editor-fold desc="在页面上放一个链接">
        """

        :param x:矩形左上角的横坐标。
        :param y:矩形左上角的纵坐标。
        :param w:矩形的宽度。
        :param h:矩形的高度。
        :param link:返回的URL或标识符加链. add_link() 方法返回值
        :return:
        """
        # 描述
        "将链接放到页面的矩形区域。文本或图像链接通常通过细胞, 写或影象，但是这个方法对于定义图像中的可点击区域很有用。"
        super().link(x=x, y=y, w=w, h=h, link=link)

        # </editor-fold>

    # 设置内部链路的目的地址
    def set_link(self, link, y=0.0, page=-1):
        # <editor-fold desc="设置内部链路的目的地址">
        """

        :param link:返回的URL或标识符加链. add_link() 方法返回值
        :param y:目标位置的纵坐标；-1表示当前位置。默认值为0(页面顶部)。
        :param page:目标页数；-1表示当前页，这是默认值。
        :return:
        """
        # 描述
        "定义链接指向的页面和位置。"
        super().set_link(link, y=y, page=page)

    # 虚线
    def dashed_line(self, x1, y1, x2, y2, dash_length=1, space_length=1):
        """
        :param x1:第一点横坐标
        :param y1:第一点纵坐标
        :param x2:第二点横坐标
        :param y2:第二点纵坐标
        :param dash_length:短跑长度
        :param space_length:破折号之间空格的长度
        :return:
        """
        # 描述
        """
        在两点之间画一条虚线。同界面线除了这两个参数，破折号和空格长度。
        """
        self._set_dash(dash_length, space_length)
        self.line(x1, y1, x2, y2)
        self._set_dash()
        """
        添加从点(10，30)开始的虚线，以点(110，30)结尾的#结束，其#破折号长度为1，空格长度为10。
        pdf.dashed_line(10, 30, 110, 30, 1, 10)
        """

    # 设定线宽
    def set_line_width(self, width: float):
        """
        定义线宽。默认情况下，该值等于0.2毫米。可以在创建第一个页面之前调用该方法，并在每个页面之间保留该值。
        :param width:宽度。
        :return:
        """
        super().set_line_width(width=width)

    # 线
    def line(self, x1, y1, x2, y2):
        """在两点之间划一条线。
        :param x1:第一点横坐标
        :param y1:第一点纵坐标
        :param x2:第二点横坐标
        :param y2:第二点纵坐标
        :return:
        添加从点(10,30)开始到点(110,30)结束的一行。
        pdf.line(10, 30, 110, 30)
        """
        super().line(x1=x1, y1=y1, x2=x2, y2=y2)

    # 画一个椭圆和圆形
    def ellipse(self, x: float, y: float, w: float, h: float, style=''):
        """
        输出椭圆。它可以绘制(仅限边界)、填充(无边界)或两者兼而有之。
        与PHP版本不同，此函数使用椭圆的左上角位置、宽度和高度，如下所示直角而不是中心点和半径。
        :param x:左上角边框的横坐标。
        :param y:左上角边框的纵坐标。
        :param w:宽度。
        :param h:高度。
        :param style:渲染风格。可能的价值是：
                    D或者是空字符串：抽签。这是默认值。
                    F*填补
                    DF或FD*绘制和填充
        :return:
        """
        super().ellipse(x=x, y=y, w=w, h=h, style=style)

    # 矩形
    def rect(self, x: float, y: float, w: float, h: float, style=''):
        """
        输出矩形。它可以绘制(仅限边界)、填充(无边界)或两者兼而有之。
        :param x:左上角的横坐标。
        :param y:左上角的纵坐标。
        :param w:宽度。
        :param h:高度。
        :param style:渲染风格。可能的价值是：
                    D或者是空字符串：抽签。这是默认值。
                    F*填补
                    DF或FD*绘制和填充
        :return:
        """
        super().rect(x=x, y=y, w=w, h=h, style=style)

    # 获取当前字体中字符串的宽度
    def get_string_width(self, s: str):
        """
        返回用户单元中字符串的长度。必须选择字体。
        该值是用拉伸和间距计算的。
        :param s:要计算长度的字符串。
        :return:
        """
        return super().get_string_width(s=s)

    # 定画颜色:所有绘图操作(线条、矩形和单元格边框)
    def set_draw_color(self, r: int, g: int = -1, b: int = -1):
        """

        :param r:如果g和b如果没有，则表示灰色级别。值介于0到255之间。
        :param g:绿色组件(介于0到255之间)。
        :param b:蓝色组件(介于0到255之间)。
        :return:
        """
        # 描述
        """
        定义用于所有绘图操作(线条、矩形和单元格边框)的颜色。
        它可以用RGB组件表示，也可以用灰度表示。可以在创建第一个页面之前调用该方法，并在每个页面之间保留该值。
        """
        super().set_draw_color(r=r, g=g, b=b)

    # 设置填充颜色:填充操作(填充矩形和单元格背景)的颜色
    def set_fill_color(self, r, g=-1, b=-1):
        """
        定义用于所有填充操作(填充矩形和单元格背景)的颜色。
        它可以用RGB组件表示，也可以用灰度表示。
        可以在创建第一个页面之前调用该方法，并在每个页面之间保留该值。
        :param r:
        :param g:
        :param b:
        :return:
        """
        super().set_fill_color(r=r, g=g, b=b)

    # 用于文本的颜色
    def set_text_color(self, r, g=-1, b=-1):
        """
        定义用于文本的颜色。
        它可以用RGB组件表示，也可以用灰度表示。
        可以在创建第一个页面之前调用该方法，并在每个页面之间保留该值。
        :param r:
        :param g:
        :param b:
        :return:
        """
        super().set_text_color(r=r, g=g, b=b)

    def text(self, x: float, y: float, txt: str = ''):
        """

        :param x:原点的横坐标。
        :param y:原点的纵坐标。
        :param txt:字符串
        :return:
        """
        # 描述
        """
        打印字符串。原点在第一个字符的左边，在基线上。
        此方法允许在页面上精确放置字符串，但通常更容易使用。细胞, 多室或写，这是打印文本的标准方法。
        """
        super().text(x=x, y=y, txt=txt)

    # 返回当前职位的横坐标。
    def get_x(self):
        return super().get_x()

    # 返回当前位置的纵坐标。
    def get_y(self):
        return super().get_y()

    # 设置纵坐标
    def set_x(self, x):
        "定义当前职位的横坐标。如果传递的值为负值，则相对于页的右侧。"
        super().set_x(x=x)

    # 设置纵坐标
    def set_y(self, y):
        "将当前的横坐标移回左侧边距，并设置纵坐标。如果传递的值为负值，则相对于页面底部。"
        super().set_y(y=y)

    # 设置纵坐标 设置纵坐标
    def set_xy(self, x, y):
        """

        :param x:横坐标的值
        :param y:纵坐标的值。
        :return:
        """
        # 定义当前位置的横坐标和纵坐标。如果传递的值为负值，则它们分别相对于页的右侧和底部。
        super().set_xy(x=x, y=y)

    def ln(self, h=''):
        """
        :param h: 休息的高度。默认情况下，该值等于上次打印单元格的高度。
        :return:
        """
        # 执行换行操作。当前的横坐标返回到左边的边距，纵坐标通过参数中传递的数量增加。

        super().ln(h=h)

    # 定义左、上和右边距。
    def set_margins(self, left: float, top: float, right: float = -1):
        # 定义左、上和右边距。默认情况下，它们等于1厘米。调用此方法来更改它们。
        super().set_margins(left=left, top=top, right=right)

    #  设上边距
    def set_top_margin(self, margin: float):
        """
        定义上边距。可以在创建第一页之前调用该方法。
        :param margin:边距
        :return:
        """
        super().set_top_margin(margin=margin)

    # 设定右边距
    def set_right_margin(self, margin: float):
        """
        定义正确的页边距。可以在创建第一页之前调用该方法。
        :param margin:边距
        :return:
        """
        super().set_right_margin(margin=margin)

    # 设定左边距
    def set_left_margin(self, margin: float):
        """
        定义左边框。可以在创建第一页之前调用该方法。如果当前的横坐标离开页面，它将返回到边距。
        :param margin:边距
        :return:
        """
        super().set_left_margin(margin=margin)

    # 保存文件
    def output(self, name='', dest=''):
        """
        :param name:文件的名称。仅在写入文件时使用。
        :param dest:发送文档的目的地。它可以采用下列值之一：
        I或D*将文件写入Sys.stdout。如果没有给出文件名，这是默认的。
        F：：保存到具有给定名称的本地文件(可能包括路径)。如果给出了文件名，这是默认的。
        S：：以字节字符串的形式返回文档。
        :return:
        """
        # 描述
        """
        将文档发送到某个目标：标准输出、文件或字节字符串。
        方法首先调用关必要时终止文档。
        注意事项：在Python中，字符串是原始数据，但在Python 3中，字符串现在默认为Unicode。
        如果您正在使用Python3.x，则必须使用pdf.output(dest='S').encode('latin-1')为了获得输出，
        如果不这样做，生成的PDF将是无效的，取决于查看器要么根本不打开，要么显示为一些空白页。
        """
        super().output(name=name,
                       dest='S'
                       ).encode('latin-1')

    # 关闭
    def close(self):
        """
        终止PDF文档。不需要显式调用此方法，因为输出量自动完成。
        如果文档不包含页面，添加页调用以防止获取无效文档。
        :return:
        """
        super().close()

    # 设定拉伸
    # def set_stretching(self,stretching: float):
    #     super().set_str


pdf = FPDF()
pdf.add_font('youyuan22', '', r'F:\ContentPro\PythonPro\automaticoffice\字体\youyuan - 副本.TTF', True)
pdf.add_page()
pdf.set_font('youyuan22', size=12)
pdf.cell(40, 10, 'Hello www我!踩踩踩踩踩踩踩踩踩踩踩踩踩踩')
pdf.output('example1.pdf', dest='F'
           )
# pdf = MyPDF(orientation='P', unit='mm', format='A4')
# pdf.add_font('youyuan', '', r'F:\ContentPro\PythonPro\automaticoffice\字体\youyuan.TTF', True)
# pdf.add_page()
# pdf.set_font("youyuan", size=12)
#
# print(pdf.w)
# print(pdf.h)
# # pdf.multi_cell(pdf.w, pdf.h,
# pdf.cell(0, 6, "ddddddddddddddddddd")
# # pdf.cell(0, 6, "华沙")
# # pdf.set_title('好好学习')
# #
# # pdf.cell(0, 6, "two", 0, ln=0, align="R")
# # pdf.cell(0, 6, "three", 0,ln=0, align="L")
# # pdf.cell(0, 6, "four", 0,ln=1, align="R")
# pdf.write(h=1000, txt='我是的是的')
# # pdf.footer()
# print(1111111111)
# pdf.output("simple_demo.pdf")
# pdf.close()
# pdf = MyPDF(orientation='P', unit='mm', format='A4')
# # author = '陈子晴'.encode('latin-1')
# # print(author)
# # pdf.set_author(author=author)
# pdf.set_creator(creator="chen")
# pdf.set_keywords('sky ocean chen ')
# pdf.set_title('dfadfasdfasd')
# pdf.output('as.pdf', 'F')  # 保存文件
# pdf.close()  # 关闭



# -*- coding: utf-8 -*-
"""
@Time : 2024/3/1 16:31 
@项目：文件使用
@File : reportlab制作pdf.by
@PRODUCT_NAME :PyCharm
"""
import datetime

from reportlab.lib import colors
from reportlab.lib.colors import HexColor
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import cm
from reportlab.lib.units import inch
# 注册中文字体  下载自己需要的.ttf字体，例如STSONG.ttf
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.pdfmetrics import registerFontFamily
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import PageTemplate
from reportlab.platypus import Paragraph, Table
from reportlab.platypus.doctemplate import BaseDocTemplate, Frame

# youyuan_ttf = os.path.join(os.path.abspath(os.path.join(os.getcwd(), "../..")), r'media\ttf\youyuan.TTF')
youyuan_ttf = r'F:\ContentPro\PythonPro\automaticoffice\字体\youyuan.TTF'
fontName = 'youyuan'

pdfmetrics.registerFont(TTFont(
    fontName,  # 命名
    youyuan_ttf,  # 字体位置
))
registerFontFamily(fontName, )


class MyReportlab():
    def footer(self, canvas, doc):
        Style = getSampleStyleSheet()
        normalStyle = Style['Normal']
        normalStyle.wordWrap = 'CJK'
        normalStyle.fontName = fontName  # 使用的字体
        """
        设置页脚
        :param canvas:Canvas类型  pdf画布
        :param doc:doc类型   整个pdf文件
        """
        canvas.saveState()  # 先保存当前的画布状态
        # pageNumber = ("%s/%s页" % (canvas.getPageNumber(), 1))  # 获取当前的页码
        pageNumber = ("%s页" % (canvas.getPageNumber(),))  # 获取当前的页码
        # pageNumber = ("%s/%s页" % (canvas.getPageNumber(), doc.pages))  # 获取当前的页码
        p = Paragraph(pageNumber, normalStyle)
        w, h = p.wrap(4 * cm, 1 * cm)  # 申请一块1cm大小的空间，返回值是实际使用的空间
        p.drawOn(canvas, (doc.width - len(pageNumber)) / 2, 10)  # 将页码放在指示坐标处
        canvas.restoreState()

    def header(self, canvas, doc):
        """
        设置页眉
        :param canvas:Canvas类型  pdf画布
        :param doc:doc类型     整个pdf文件
        """
        Style = getSampleStyleSheet()
        normalStyle = Style['Normal']
        normalStyle.wordWrap = 'CJK'
        normalStyle.fontName = fontName  # 使用的字体
        canvas.saveState()
        # p = Paragraph("<img src='%s' width='%d' height='%d'/>" % ('img_address', 11, 12),
        #               normalStyle)  # 使用一个Paragraph Flowable存放图片
        p = Paragraph("11111111111sdasdsada",
                      normalStyle)  # 使用一个Paragraph Flowable存放图片
        w, h = p.wrap(doc.width, doc.bottomMargin)
        p.drawOn(canvas, doc.leftMargin, doc.topMargin + doc.height - 0.5 * cm)  # 放置图片
        # p = Paragraph("<font size=10 face='STSong-Light'>报告</font>", normalStyle)
        # w, h = p.wrap(doc.width, doc.bottomMargin)
        # p.drawOn(canvas, doc.leftMargin + doc.width - 2.2 * cm, doc.topMargin + doc.height - 0.3 * cm)  # 放置报告这句话
        canvas.line(doc.leftMargin, doc.bottomMargin + doc.height + 0.5 * cm, doc.leftMargin + doc.width,
                    doc.bottomMargin + doc.height + 0.5 * cm)  # 画一条横线
        canvas.restoreState()

    def table_model_all(self, data):
        width = 10  # 总宽度
        colWidths = (width / len(data[0])) * inch  # 每列的宽度
        dis_list = []
        for x in data:
            # dis_list.append(map(lambda i: Paragraph('%s' % i, cn), x))
            dis_list.append(x)
        style = [
            ('FONTNAME', (0, 0), (-1, -1), fontName),  # 字体
            ('FONTSIZE', (0, 0), (-1, -1), 9.1),  # 字体大小
            ('ALIGN', (0, 0), (-1, -1), 'CENTER'),  # 水平居中对齐
            ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),  # 垂直居中
            ('hAlign', (0, 0), (-1, -1), 'CENTER'),  # 对齐
            # ('BOX', (0, 0), (-1, -1), 1, colors.black),  # 外边框
            # ('INNERGRID', (2, 0), (2, -1), 0, colors.white),  # 内边框
            # ('LEADING', (2, 0), (2, -1),24),  # 行距
            ('GRID', (0, 0), (-1, -1), 1, colors.black),  # 设置表格框线为grey色，线宽为0.5
            # ('SPAN', (2, 0), (2, -1)),  # 合并 （'SPAN',(第一个方格的左上角坐标)，(第二个方格的左上角坐标))，合并后的值为靠上一行的值，按照长方形合并
            ('TEXTCOLOR', (0, 0), (-1, -1), colors.black),  # 设置表格内文字颜色
            # ('BACKGROUND', (0, 0), (-1, 0), HexColor('#d5dae6')),  # 设置第一行背景颜色
            # ('BACKGROUND', (0, 1), (-1, 1), HexColor('#d5dae6')),  # 设置第二行背景颜色
            # ('LINEBEFORE', (0, 0), (0, -1), 1, colors.grey),  # 设置表格边左边线颜色为灰色，线宽为0.1
            # ('LINEBELOW', (0, -1), (-1, -1), 2, colors.red),  # 设置表格边下边线颜色为灰色，线宽为0.1
            # ('LINEABOVE', (0, 0), (-1, 0), 2, colors.green),#设置表格上边线颜色为灰色，线宽为0.1
            # ('LINEABOVE', (0, 1), (-1, -1), 0.25, colors.red),#设置表格下边线颜色为灰色，线宽为0.1
            # ('NOSPLIT', (1, 0), (3, 18)),#间隔
            #
        ]

        component_table = Table(dis_list,
                                colWidths=[2.5 * inch, 2.5 * inch, 50, 2.5 * inch, 2.5 * inch, ],
                                style=style,
                                # vAlign="LEFT",
                                # hAlign="LEFT"
                                )

        return component_table

    def font_create(self, font_style="Normal", fontName="youyuan", wordWrap='CJK', firstLineIndent=32, leading=None,
                    fontSize=None, alignment=0, textColor=colors.black
                    ):
        Style = getSampleStyleSheet()
        font = Style[font_style]  # 字体的样式
        font.fontName = fontName  # 使用的字体
        font.wordWrap = wordWrap  # 该属性支持自动换行，'CJK'是中文模式换行，用于英文中会截断单词造成阅读困难，可改为'Normal'
        if firstLineIndent:
            font.firstLineIndent = firstLineIndent  # 该属性支持第一行开头空格
        if leading:
            font.leading = leading  # 该属性是设置行距
        if fontSize:
            font.fontSize = fontSize
        if alignment or alignment == 0:
            font.alignment = alignment  # 居中
        if textColor:
            font.textColor = colors.black  # 颜色
        return font

    def table_normal(self):
        table_style = [
            ('SPAN', (0, 0), (-1, 0)),
            ('BACKGROUND', (0, 0), (-1, 0), HexColor('#d5dae6')),
            ('FONTNAME', (0, 0), (-1, -1), fontName),  # 字体
            ('FONTSIZE', (0, 0), (-1, -1), 9.1),  # 字体大小
            ('ALIGN', (0, 0), (-1, -1), 'CENTER'),  # 水平居中对齐
            ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),  # 垂直居中
            ('hAlign', (0, 0), (-1, -1), 'CENTER'),  # 对齐
            ('GRID', (0, 0), (-1, -1), 1, colors.black),  # 设置表格框线为grey色，线宽为0.5
            ('TEXTCOLOR', (0, 0), (-1, -1), colors.black),  # 设置表格内文字颜色

        ]
        return table_style


class MyBaseDocTemplate(MyReportlab):
    def __init__(self, report_addresses):
        self.doc = BaseDocTemplate(report_addresses,
                                   pagesize=A4,
                                   # pagesize=landscape(letter),
                                   rightMargin=10,
                                   leftMargin=10,
                                   topMargin=50,
                                   bottomMargin=20
                                   )
        self.font1 = self.font_create(alignment=1, fontSize=20)
        self.font2 = self.font_create(alignment=0, firstLineIndent=32)


# 案例
class HwdReport(MyBaseDocTemplate):
    def __init__(self, report_addresses):
        super().__init__(report_addresses)

    def get_fwd_build(self, date=datetime.datetime.now().strftime('%Y年%m月%d日'),
                      area="同济大学嘉定校区通达馆副楼外",
                      creator="佚名",
                      creator_sign="",
                      test_by="同科",
                      brand="熙赢测控",
                      version="SHN-HWD-S",
                      number="SHN-HWD-S（无）",
                      operators="匿名"):
        build_lis = [
            Paragraph('HWD标定报告', self.font1),
            Paragraph(f'<br/><br/><br/>', self.font2),
            Paragraph(f'\t标定日期:{"&emsp;" * 4}{date}', self.font2),
            Paragraph(f'\t标定场地:{"&emsp;" * 4}{area}', self.font2),
            Paragraph(f'\t标定人员:{"&emsp;" * 4}{creator}', self.font2),
            Paragraph(f'\t标定人员签名:_________________', self.font2),
            Paragraph(f'<br/><br/><br/>', self.font2),

            Paragraph(f'\tHWD测试方:{"&emsp;" * 4}{test_by}', self.font2),
            Paragraph(f'\tHWD品牌:{"&emsp;" * 5}{brand}', self.font2),
            Paragraph(f'\tHWD型号:{"&emsp;" * 5}{version}', self.font2),
            Paragraph(f'\tHWD编号:{"&emsp;" * 5}{number}', self.font2),
            Paragraph(f'\tHWD操作人员:{"&emsp;" * 3}{operators}', self.font2),
            Paragraph(f'<br/><br/><br/>', self.font2),
            Paragraph(f'\tHWD随机误差', self.font_create(alignment=0, firstLineIndent=32)),

        ]
        table1_content = [
            [
                ['HWD荷载随机误差判定#1', ],
                ['判定阈值', ''],
                ['误差大小', '002'],
                ['判定结果', '003'],
                ['误差来源', '003\n005']
            ],
            [
                ['HWD Geophone随机误差判定#1', ''],
                ['判定阈值', ''],
                ['误差大小', 'Geophone随机误差判定Geophone随机误差判定Geophone随机误差判定\nGeophone随机误差判定'],
                ['判定结果', '003'],
                ['误差来源', '003\n005']
            ]]
        for table_data in table1_content:
            build_lis.append(Paragraph(f'<br/><br/><br/>', self.font2), )
            build_lis.append(
                Table(table_data,
                      colWidths=[2.5 * inch, 5 * inch],
                      style=self.table_normal(),
                      # vAlign="LEFT",
                      # hAlign="LEFT"
                      )
            )
        build_lis.append(Paragraph(f'<br/><br/><br/>', self.font2), )
        build_lis.append(
            Paragraph(f'\t标定模型', self.font_create(alignment=0, firstLineIndent=32)),
        )
        table2_content = [
            [
                ['Geophone #1', 'Geophone #1'],
                ['距离承载板', ''],
                ['标定方程', '002'],
                ['决定系数（R2）', '003'],
                ['误差来源', '003\n005']
            ],
            [
                ['Geophone #2', 'Geophone #2'],
                ['距离承载板', ''],
                ['标定方程', '002'],
                ['决定系数（R2）', '003'],
                ['误差来源', '003\n005']
            ],
            [
                ['Geophone #3', 'Geophone #3'],
                ['距离承载板', ''],
                ['标定方程', '002'],
                ['决定系数（R2）', '003'],
                ['误差来源', '003\n005']
            ],
            [
                ['Geophone #4', 'Geophone #4'],
                ['距离承载板', ''],
                ['标定方程', '002'],
                ['决定系数（R2）', '003'],
                ['误差来源', '003\n005']
            ],
            [
                ['Geophone #5', 'Geophone #5'],
                ['距离承载板', ''],
                ['标定方程', '002'],
                ['决定系数（R2）', '003'],
                ['误差来源', '003\n005']
            ],
            [
                ['Geophone #6', 'Geophone #6'],
                ['距离承载板', ''],
                ['标定方程', '002'],
                ['决定系数（R2）', '003'],
                ['误差来源', '003\n005']
            ],
        ]
        for table_data in table2_content:
            build_lis.append(Paragraph(f'<br/><br/><br/>', self.font2), )
            build_lis.append(
                Table(table_data,
                      colWidths=[2.5 * inch, 5 * inch, ],
                      style=self.table_normal(),
                      # vAlign="LEFT",
                      # hAlign="LEFT"
                      )
            )
        build_lis.append(Paragraph(f'<br/><br/><br/>', self.font2), )

        build_lis.append(Paragraph(f'标定结论', self.font2), )
        build_lis.append(
            Paragraph(f'测试方 {test_by}  来标定的HWD品牌为 {brand} ，型号为 {number}，接受机场道面适航安全测试设备标定管理平台的标定', self.font2),
        )
        build_lis.append(
            Paragraph(f'标定结论和依据如下：', self.font2),
        )
        lis = ["""（1）标定中 通过 荷载随机误差判定（误差 < 1%）""",
               """（2）标定中 通过 Geophone随机误差判定（误差 < 2%）""",
               """（3）标定中 通过 系统标定，且标定方程的拟合优度良好（各决定系数 > 0.9, 决定系数均值 > 0.95）""",
               """（4）标定结论为HWD设备 通过 本次标定。""",
               ]
        for li in lis:
            build_lis.append(
                Paragraph(li, self.font2),
            )
        build_lis.append(
            Paragraph(f'根据标定数据和结果给出以下建议：', self.font2),
        )
        lis2 = [
            """（1） 标定的Geophone#3需要进行检查或组件标定，考虑替换此传感器或支架。""",
            """（2） 标定的Geophone#5需要进行检查，同时需检查本段支架。""",
            """（3） 标定的Geophone#6需要进行检查，同时需检查本段支架。  """
        ]
        for li in lis2:
            build_lis.append(
                Paragraph(li, self.font2),
            )

        return build_lis

    def get_report(self, build_data):
        frame_footer = Frame(self.doc.leftMargin, self.doc.bottomMargin, self.doc.width, self.doc.height,
                             id='normal')  # 声明一块Frame，存放页码
        template = PageTemplate(id='test', frames=frame_footer,
                                # onPage=self.header,
                                onPageEnd=self.footer)  # 设置页面模板，在加载页面时先运行herder函数，在加载完页面后运行footer函数
        self.doc.addPageTemplates([template])
        self.doc.build(build_data)


if __name__ == '__main__':
    pdf_obj = HwdReport(f"LoaderReport.pdf")
    pdf_obj.get_report(pdf_obj.get_fwd_build())



# -*- coding: utf-8 -*-
"""
@Time : 2023/8/11 18:29 
@Author : skyoceanchen
@TEL: 18916403796
@项目：file_operations
@File : configparser_operation.by
@PRODUCT_NAME :PyCharm
"""
# media/test.cfg
import configparser


class ConfigParserUtil(object):
    def __init__(self, path):
        self.path = path
        # 创建一个解析器
        self.config = configparser.ConfigParser()
        # 读取并解析test.cfg
        self.config.read(self.path, encoding='utf-8')

    def read(self):
        pass

    # 获取分区
    def sections(self):
        return self.config.sections()

    # 获取选项
    def options(self, section):
        return self.config.options(section)

    # 获取某个选项的值
    def option_value(self, section, option, data_type=None):
        """
        :param section:
        :param option:
        :param data_type:int float db
        :return:
        """
        if data_type == "int":
            return self.config.getint(section, option)
        elif data_type == "float":
            return self.config.getfloat(section, option)
        elif data_type == "bool":
            return self.config.getboolean(section, option)
        else:
            return self.config.get(section, option)

    # 是否有某个分区
    def has_section(self, section):
        return self.config.has_section(section)

    # 是否有某个选项
    def has_option(self, section, option):
        self.config.has_option(section, option)

    # # 添加分区
    def add_section(self, section):
        if not self.config.has_section(section):
            self.config.add_section(section)
        else:
            raise Exception("分区已存在")

    # # 删除选项
    def remove_option(self, section, option):
        self.config.remove_option(section, option)

    #  添加/  修改
    def set_option(self, section, option, value):
        self.config.set(section, option, str(value))

    # # 增删改查操作完成后写回文件中
    def save(self):
        with open(self.path, "wt", encoding="utf-8") as f:
            self.config.write(f)


if __name__ == '__main__':
    """
    config_obj = ConfigParserUtil('mqtt.cfg')
    # config_obj.add_section('mqtt')
    config_obj.set_option('mqtt', "a", '1a')
    config_obj.save()
    print(config_obj.sections())
    print(type(config_obj.option_value('mqtt', 'a')))
    """
    pass



# -*- coding:utf-8 -*-
"""
@Time : 2023/3/22
@Author : skyoceanchen
@TEL: 18916403796
@File : other_operation.py 
@PRODUCT_NAME : PyCharm 
"""


class ReadDat(object):
    def read_dat(self, path):
        with open(path, 'r', encoding='utf-8') as f:
            text = f.read()
        return text
